cd('~/Dropbox/Amazon_FC/')
clear;

% Project: Nexus Tax Laws and Economies of Density in E-Commerce: A Study of Amazon’s Fulfillment Center Network
% Authors: Houde, Newberry, Seim
% Created by: Peter Newberry
% Date: 05/2022
% Description: Generates the mode-level FEs using the estimated model and
% the revenue data from Amazon's financial statements as well as the US
% dept of commerce
% Data sets needed: projected_fe_wf.csv: created in table3.do. 

data=csvread('Data/Processed/Intermediate/projected_fe_wf.csv');

mode=data(:,1);
year=data(:,2);
county=data(:,3);
delta=data(:,4);
mode_fe=data(:,5);
spending=data(:,6);
tax_rhs=data(:,7); %no longer use
adjust=data(:,8); %adjustment from tax inclusive 
am_sales=data(:,9); % am sales from financial statments
mode2_sales=data(:,10); %mode 2 sales from US dept of commerce weighted by shares in comScore data.
mode3_sales=data(:,11); %mode 3 sales from US dept of commerce weighted by shares in comScore data.
hh=data(:,12); %number of hh



out_data=[];

% find the set of mode FEs that match the data the closest. Years within
% our sample.
for y=1999:1:2016
year2=y;
data2=data;
data2((data2(:,2)~=year2),:)=[];

options = optimoptions(@fminunc, 'Display', 'iter', 'MaxFunEvals', 100000, 'OptimalityTolerance', .000000000001);

efunc=@ (x) obj_func_predicFE_lin(x, data2);
start=[0, 0, 0];
[x,fval2,exitflag,output]=fminunc(efunc, start, options);
out_data=[out_data; [y, 1, x(1)];[y, 2, x(2)];[y, 3, x(3)]];
end

% find the set of mode FEs that match the data the closest. Years outside
% of
% our sample.
for y=2017:1:2018
year2=y;
data2=data;
data2((data2(:,2)~=year2),:)=[];

options = optimoptions(@fminunc, 'Display', 'iter', 'MaxFunEvals', 100000, 'OptimalityTolerance', .000000000001);

efunc=@ (x) obj_func_predicFE_lin(x, data2);
start=[0, 0, 0];
[x,fval2,exitflag,output]=fminunc(efunc, start, options);
out_data=[out_data; [y, 1, x(1)];[y, 2, x(2)];[y, 3, x(3)]];
end

dlmwrite('Data/Processed/Intermediate/projected_fe.csv', out_data, 'delimiter', ",", 'precision', 12)

