function out=obj_func_predicFE_lin(x, data)


% Project: Nexus Tax Laws and Economies of Density in E-Commerce: A Study of Amazon’s Fulfillment Center Network
% Authors: Houde, Newberry, Seim
% Created by: Peter Newberry
% Date: 05/2022
% Description: Function that takes given FEs and calculates total spending
% for each mode. Then we form an objective funciton based on the difference
% betweent the predicted spending and the observed spending.

mode=data(:,1);
year=data(:,2);
county=data(:,3);
delta=data(:,4);
mode_fe=data(:,5);
spending=data(:,6);
tax_rhs=data(:,7); %no longer use
adjust=data(:,8); %adjustment from tax inclusive 
am_sales=data(:,9); % am sales from financial statments
mode2_sales=data(:,10); %mode 2 sales from US dept of commerce weighted by shares in comScore data.
mode3_sales=data(:,11); %mode 3 sales from US dept of commerce weighted by shares in comScore data.
hh=data(:,12); %number of hh

% calculate LHS of regression given FEs
exp_lhs=exp(delta+x(1)*(mode==1)+x(2)*(mode==2)+x(3)*(mode==3));


% create total spending across 3 only modes in each county;
create_index=unique([county], 'rows');
index=1:length(create_index);
index=index';
[tf,loc]=ismember([county], create_index, 'rows');
indices=[index(loc(tf),:)];
tot_Y=accumarray(indices,exp_lhs,[numel(index),1], @sum);
[tf,loc]=ismember([county], create_index, 'rows');
county_exp_lhs= tot_Y(loc(tf),:);

% predict county level spending on each mode
offline_spend=spending*1./(county_exp_lhs+1);
pre_spend=exp_lhs.*offline_spend;

pre_net_spend=adjust.*pre_spend;

% get total spending by mode
create_index=unique([mode], 'rows');
index=1:length(create_index);
index=index';
[tf,loc]=ismember([mode], create_index, 'rows');
indices=[index(loc(tf),:)];
mode_spending=accumarray(indices,hh.*pre_net_spend,[numel(index),1], @sum)./1000000;

observed_am=unique(am_sales);
observed_mode2=unique(mode2_sales);
observed_mode3=unique(mode3_sales);

observed=[observed_am; observed_mode2; observed_mode3];

%objective function of predicted spending and observed. 
out=sum((mode_spending-observed)'*(mode_spending-observed));

end