capture log close
set more off
clear
// Project: Nexus Tax Laws and Economies of Density in E-Commerce: A Study of Amazon’s Fulfillment Center Network
// Authors: Houde, Newberry, Seim
// Created by: Peter Newberry
// Date: 05/2022
// Description: Creates the inflation factors to blow up spending to match observed data.
// Data sets needed: Total_Spending_99_17.xlsx: total spending on ecommerce and Amazon.
//					 Amazon_finances.xlsx: financial information for Amazon
//					 Panel_county_tax_dist.dta: has county level tax information as well as information on distances to FCs and SCS. The tax information is propreitary. Souces is TDS (tax data systems).It also has whether or not transacitons in the year/mode are taxable. 
//					 comscore_population_weight.dta: population weights used to aggregate data. calculated in population_weights.do
//					 hh_expenditures.dta: household expenditure data from comscore, calculated in hh_expenditure.do
// Author notes: based on author do-file "inflation_factor_calc.do"

*****************************************************************************
*****************************************************************************
*****************************************************************************
*****************************************************************************
*****************************************************************************
*****************************************************************************
*****************************************************************************
*****************************************************************************
******************************************************************************
******************************************************************************
******************************************************************************
//First part imports spending files to get "observed" spending information on the differnt modes as well as the 3rd party share and the margins for Amazon and 3rd party sales.

import excel "Total_Spending_99_17.xlsx", cellrange(A17:H37) firstrow clear
keep year totalEcommerce
sort year
save "observed_spending.dta", replace


import excel "Amazon_finances.xlsx", cellrange(a1:s21) firstrow clear sheet(Updated)
preserve
keep Year thirdpartyshare
rename Year year
rename thirdpartyshare mp_share
sort year
save "mp_share.dta", replace
restore
keep Year PredictAmazonSales PredictMPSales PredictTotalSales
rename Year year
rename PredictAmazonSales direct_sales
rename PredictMPSales mp_sales
rename PredictTotalSales AmazonSales
sort year
save "observed_spending2.dta", replace


import excel "Amazon_finances.xlsx", cellrange(A1:AZ19) firstrow clear sheet(Primary_Data)
keep Year NAgrossmargin ReportedGrossMargin ReportedGrossRelMargin ReportedGrossMarginNA ReportedGrossRelMarginNA
rename Year year
rename NAgrossmargin margin0
rename ReportedGrossMargin margin_reported
rename ReportedGrossRelMargin margin_reported_ship
rename ReportedGrossMarginNA margin_reported_us
rename ReportedGrossRelMarginNA margin_reported_ship_us
sort year
save "margins.dta", replace


*****************************************************************************
*****************************************************************************
*****************************************************************************
*****************************************************************************
*****************************************************************************
*****************************************************************************
*****************************************************************************
*****************************************************************************
******************************************************************************
******************************************************************************
******************************************************************************
//Second part agggregates up the predicted spending using our weights and then finds the "inflation factor" needed to blow up expenditures to match the aggregate expenditures observed on Financial statments and via the US dept of commerce.

use "hh_expenditures.dta", clear

//merge in various data sets
rename county county_code
merge m:1 county_code year using "Panel_county_tax_dist.dta"
tab _merge
keep if _merge==3
drop _merge
rename county_code county


sort county race age income_group year
merge county race age income_group year using "comscore_population_weight.dta"
keep if _merge==3
drop _merge


// adjust expenditures for Amazon and mode2 because the expenditures include tax and the figures we are matching (presumably) don't.
replace tot_exp1_forr=tot_exp1_forr*1/(1+amazon_tax*(1-mp_share))
replace tot_exp2_forr=tot_exp2_forr*1/(1+tax)

// generate total US sales for each mode
egen US_exp_forr_county1=sum(tot_exp1_forr*weight_county1), by(year)
egen US_exp_forr_county2=sum(tot_exp2_forr*weight_county1), by(year)
egen US_exp_forr_county3=sum(tot_exp3_forr*weight_county1), by(year)

//generate total US online spending
gen US_exp_forr_county=US_exp_forr_county1+US_exp_forr_county2+US_exp_forr_county3

//generate the per household sales for each mode and overall
gen ave_exp_forr_county1=US_exp_forr_county1/tot_hh_us_county
gen ave_exp_forr_county2=US_exp_forr_county2/tot_hh_us_county
gen ave_exp_forr_county3=US_exp_forr_county3/tot_hh_us_county

gen ave_exp_forr_county=US_exp_forr_county/tot_hh_us_county

contract year ave_* US_* tot_hh_* per_*

// merge in "observed" spending data
sort year
merge year using "observed_spending.dta"
drop _merge


sort year
merge year using "observed_spending2.dta"
drop _merge

// rename and adjust scale
rename AmazonSales amazon_sales
gen ave_observed_amazon_sales=1000000*amazon_sales/tot_hh_us_true

rename totalEcommerce ecommerce_sales

// create household average sepnding on the modes 
gen ave_observed_other_sales=1000000*(ecommerce_sales-amazon_sales)/tot_hh_us_true
gen ave_observed_other_county_forr=(US_exp_forr_county2+US_exp_forr_county3)/tot_hh_us_county 

// find the percentage of spending that is "missing" in comscore.
gen missing_perc_amazon_county_forr=(ave_observed_amazon_sales-ave_exp_forr_county1)/ave_observed_amazon_sales
gen missing_perc_all_county_forr=(ave_observed_other_sales-(ave_exp_forr_county2+ave_exp_forr_county3))/ave_observed_other_sales


drop if year>2016 | year<2006
keep year missing*
sort year
save "infaltion_factors.dta", replace

