capture log close
set more off
clear
// Project: Nexus Tax Laws and Economies of Density in E-Commerce: A Study of Amazon’s Fulfillment Center Network
// Authors: Houde, Newberry, Seim
// Created by: Peter Newberry
// Date: 05/2022
// Description: Creates transaction level data sets from the RAW comscore data. For each data set, we eliminate different categories and/or websites based on what goods they sell. 
// Data sets needed: competitive_sites.dta: this lists the websites in the comScore data and we determine if each site is a competitor of Amazon or not. Since this comes from comScore, it is propreitary. 
//					 domain_mode_match.dta: a data set with each website in the comscore data and we determine if each site has a broad offline presence and assign it to eaither mode 2 or 3. Since this comes from comScore, it is propreitary. 
//					 TransactionsRaw.csv: the RAW comscore transaction data from 2004-2016. Proprietary.

// Author notes: based on author do-file "transaction_data_create.do"



// import raw comScore data
insheet using "TransactionsRaw.csv", clear names comma

// drop 2004 an 2005
drop if year<2006

// merge in mode definition
sort domain_name
merge domain_name using "domain_mode_match.dta"
replace mode=3 if mode==.
drop _merge

// save a data set with all transactions
save "transactions_all.dta", replace

// merge in variable that indicates if the given domain is a competitor of Amazon.
sort domain_name
sort domain_name
merge domain_name using "competitive_sites.dta"
tab _merge
drop _merge

// drop non-competitors
drop if competitor==0

// drop certain categories. 
drop if prod_category_id==0
drop if prod_category_id==18
drop if prod_category_id==41
drop if prod_category_id==42
drop if prod_category_id==43
drop if prod_category_id==44
drop if prod_category_id==45
drop if prod_category_id==46
drop if prod_category_id==47
drop if prod_category_id==48
drop if prod_category_id==49
drop if prod_category_id==50
drop if prod_category_id==51
drop if prod_category_id==52
drop if prod_category_id==53
drop if prod_category_id==57
drop if prod_category_id==58
drop if prod_category_id==59
drop if prod_category_id==377

// save main transaction data set

save "transactions_sub_dom.dta", replace







