qstat(const typeid,const vP)
{
  decl mXb=mX*vP;
  decl mM_obs=mZdotY-mZ.*mXb;
  decl vMbar_obs=vZY-mZX*vP;
  decl vD=varc(mM_obs);
  decl mInvD_sqrt=diag(vD.^(-1/2));
  decl vMbar_b,vV,vL;
  decl n=rows(vY);
  decl b=1;
  decl kappa=sqrt(log(n/b));
  decl p=columns(mZ);
  decl id_slack1,id_slack0;
  if(typeid==0) {
    vV=sqrt(n)*mInvD_sqrt*vMbar_obs;
    vL=zeros(p,1);
  }  
  if(typeid==1) {
    vV=sqrt(n)*mInvD_sqrt*((mM_obs'-vMbar_obs)*mU[][simid]/n);
    vL=(kappa^(-1))*sqrt(n)*mInvD_sqrt*vMbar_obs; /* Adjusted T-stat */
    id_slack1=vecindex(vL.>1,1);
    id_slack0=vecindex(vL.<=1,1);
    if(id_slack1!=<>) vL[id_slack1]=10000000;
    if(id_slack0!=<>) vL[id_slack0]=0;
  } 
  if(typeid==2) {
    vV=sqrt(n)*mInvD_sqrt*((mM_obs'-vMbar_obs)*mU[][simid]/n);
    vL=(kappa^(-1))*sqrt(n)*mInvD_sqrt*vMbar_obs; /* Adjusted T-stat */
  }
  return sumc(setbounds(vV+vL,M_INF_NEG,0).^2);
}
qfunc_obs(const vP, const adFunc, const avScore, const amHessian)
{
  decl n;
  decl vParam=vParam0;
  vParam[OPTid]=vP;
  vParam.*=(vParam_upper-vParam_lower);
  decl Q=qstat(0,vParam);
  adFunc[0]=double(-Q);
  return 1;
}
qfunc_dr(const vP, const adFunc, const avScore, const amHessian)
{
  decl n;
  decl vParam=vParam0;
  vParam[OPTid]=vP;
  vParam.*=(vParam_upper-vParam_lower);
  decl Q=qstat(1,vParam);
  adFunc[0]=double(-Q);
  return 1;
}
qfunc_pr(const vP, const adFunc, const avScore, const amHessian)
{
  decl n;
  decl vParam=vParam0;
  vParam[OPTid]=vP;
  vParam.*=(vParam_upper-vParam_lower);
  decl Q=qstat(2,vParam);
  adFunc[0]=double(-Q);
  return 1;
}
TestMR(const aTest,const aParam,const vParam,const alpha)
{
  decl time0;
  decl p=columns(mZ);
  decl n=rows(vY);
  decl vParam_transform=vParam./(vParam_upper-vParam_lower);
  decl vP=vParam_transform[OPTid];
  decl K=rows(OPTid);
  vParam0=vParam_transform;
  /* Starting values */
  decl vPstart_sample;
  if(aParam[0]!=<>) vPstart_sample=aParam[0];
  else vPstart_sample=vP;
  /* Declare function objects */
  decl Tdr,Tpr,Tmr,Tn;
  decl vP_sample;
    
  /***********************************************************************************************/
  /* Optimize function: Data moments */
  decl Q,iConv,r;
  decl grid=columns(mRandomGrid);
  decl mGridParam=new matrix[K][grid];
  decl vGridT=new matrix[grid][1];
  decl i,vDiff;
  decl eps1=10^(-6);
  decl eps2=eps1*10;
  decl vParam_id,mTheta_set,grid_theta;      
  time0=timer();
  /* Solve optimization problem of "grid" random starting values to construct set */
  if(K>0) {
    parallel for(r=0;r<grid;r++)
      {
	if(r==0) vP=vPstart_sample;
	else vP=mRandomGrid[OPTid][r];
	MaxControl(20,0);
	iConv=MaxSimplex(qfunc_obs,&vP,&Q,constant(1/2,vP));
	MaxControl(100,0);
	iConv=MaxBFGS(qfunc_obs,&vP,&Q,0,1);
	if(maxc(fabs(vP))>100) {
	  iConv=MAX_NOCONV;
	  vP=constant(.NaN,vP);
	  Q=.NaN;
	  vGridT[r]=.NaN;
	}
	if(iConv<=MAX_WEAK_CONV) {
	  vGridT[r]=-Q;
	  mGridParam[][r]=vP;
	}
	else {
	  vGridT[r]=.NaN;
	  mGridParam[][r]=vP;
	}
      }
    vGridT=deleteifr(vGridT,sumc(mGridParam)'.==.NaN);
    mGridParam=deletec(mGridParam);
    Tn=minc(vGridT);
    if(Tn!=.NaN) {
      /* Optimization set: Distinct parameters corresponding to the minimizing solution */
      vParam_id=vecindex(fabs(vGridT-Tn).<eps1,1);
      mTheta_set=mGridParam[][vParam_id[0]];
      for(i=0;i<rows(vParam_id);i++)
	{
	  vDiff=maxc(fabs((mTheta_set-mGridParam[][vParam_id[i]])./mGridParam[][vParam_id[i]]))';
	  if(minc(vDiff)>eps2) mTheta_set~=mGridParam[][vParam_id[i]];
	}
      grid_theta=columns(mTheta_set);
      mTheta_set=sortbyc(mTheta_set',range(0,K-1))';
      vP_sample=mGridParam[][floor(grid_theta/2)]; /* Parameter: Mid-point of optimizing set */
      if(iprint) println("vP (sample) : ",vP_sample);
      if(iprint) println("Tn : ",Tn);
      if(iprint) println("cpu time (Tn): ",(timer()-time0)/100);  
      if(iprint) println("nb of elements: ",grid_theta~rows(vParam_id));
    }
  }
  else {
    qfunc_obs(<>,&Tn,0,0);
    Tn=-Tn;
    vP_sample=<>;
    grid_theta=0;
  }
  if(Tn!=.NaN) {
    /***********************************************************************************************/  
    /* Optimize function: Bootstrap sample */
    MaxControl(100,0);
    time0=timer();
    decl typeid;    
    decl B=5000;
    decl vTmr_sim=new matrix[B][1];
    decl vPstart=vP_sample;
    decl time00=timer();
    decl maxit=grid;
    simid=0;
    decl b=0;
    do{
      time0=timer();
      for(typeid=1;typeid<=2;typeid++)
	{
	  if(typeid==1) {
	    /* DR Test */
	    vGridT=new matrix[grid_theta][1];
	    mGridParam=new matrix[K][grid_theta];
	    if(K>0) {
	      for(r=0;r<grid_theta;r++)
		{
		  vP=mTheta_set[][r];
		  qfunc_dr(vP,&Q,0,0);
		  if(Q!=.NaN) {
		    vGridT[r]=-Q;
		    mGridParam[][r]=vP;
		  }
		  else {
		    vGridT[r]=M_INF_POS;
		    mGridParam[][r]=vP;
		  }
		}	  
	      Tdr=minc(vGridT);
	    }
	    if(K==0) {
	      vP=<>;
	      qfunc_dr(vP,&Q,0,0);
	      Tdr=-Q;
	      r=0;
	    }
	    if(iprint) println("DR test results = ",r~Tdr);
	  }
	  if(typeid==2) {
	    /* PR Test */
	    if(K>0){
	      r=0;
	      do{
		if(r>0) vP=mRandomGrid[OPTid][r];
		if(r==0) vP=vPstart;
		MaxControl(20,0);
		iConv=MaxSimplex(qfunc_pr,&vP,&Q,constant(1/2,vP));
		MaxControl(20,0);
		iConv=MaxBFGS(qfunc_pr,&vP,&Q,0,1);
		if(maxc(fabs(vP))>100) iConv=MAX_NOCONV;
		if(iConv>MAX_WEAK_CONV) Q=.NaN;
		r+=1;
	      }while(iConv>MAX_WEAK_CONV && r<10);
	      if(isnan(vPstart)==0) vPstart=vP;
	      Tpr=-Q;
	    }
	    if(K==0){
	      vP=<>;
	      qfunc_pr(vP,&Q,0,0);
	      Tpr=-Q;
	      r=0;
	      iConv=MAX_CONV;
	    }
	    if(iprint) println("PR test results = ",r~Tpr~iConv);
	  }
	}
      Tmr=minc(Tdr|Tpr);
      if(isnan(Tdr|Tpr)==0) {
	vTmr_sim[b]=Tmr;
	b+=1;
      }
      else Tmr=.NaN;
      if(iprint) println("cpu time (DR+PR test): ",(timer()-time0)/100);
      if(iprint) println("Tmr: ",Tmr);
      simid+=1;
    }while(b<B);
    println("total CPU time: ",(timer()-time00));
    vTmr_sim=deleter(vTmr_sim);
    println("dim vTmr_sim: ",rows(vTmr_sim)~columns(vTmr_sim));
    aTest[0]=Tn~quantilec(vTmr_sim,1-alpha);
    aParam[0]=vP_sample;
  }
  else {
    aTest[0]=.NaN;
    aParam[0]=.NaN;
  }
  return 1;
}