#delimit;
global computer "Local";
*global computer "Linstat";
if "$computer" == "Local" {;
	local intermediate_dir "~/Dropbox/Amazon_FC/Data/Processed/Intermediate";
	local swap_dir "~/Dropbox/Amazon_FC/Data/Processed/Swap_list";
	local main_dir "~/Dropbox/Amazon_FC/Data/Processed/Main";
	local county_dir "~/Dropbox/Amazon_FC/Data/County_Characteristics";
	local FC_dir "~/Dropbox/Amazon_FC/Data/FCs";
	};
if "$computer" == "Linstat" {;
	local intermediate_dir "/project/houde/Amazon_FC/Data/Processed/Intermediate";
	local swap_dir "/project/houde/Amazon_FC/Data/Processed/Swap_list";
	local main_dir "/project/houde/Amazon_FC/Data/Processed/Main";
	local county_dir "/project/houde/Amazon_FC/Data/County_Characteristics";
	local FC_dir "/project/houde/Amazon_FC/Data/FCs";
	};
set more off;

/*
     +-----------------------------------------+
     |                      full_type   typeid |
     |-----------------------------------------|
  1. |                     FC (Fresh)        1 |
  2. |              FC (Non-Sortable)        2 |
  3. |                     FC (Prime)        3 |
  4. |                  FC (Sortable)        4 |
  5. |            Other (Call center)        5 |
     |-----------------------------------------|
  6. |                    Other (IDX)        6 |
  7. |                Other (Returns)        7 |
  8. |   Sortation (Delivery station)        8 |
  9. | Sortation (Sortation facility)        9 |
     +-----------------------------------------+
*/

/******************************************************************************/
/* This is to verify that the distance between FCs and SC is not too large */
/* Create density measure */
use "`county_dir'/demo_panel_final", clear;
keep if year==2018;
rename lat county_lat;
rename lon county_long;
collapse (mean) county_lat county_long pop_density nb_hh, by(county_code);
egen sum_density=sum(pop_density*nb_hh);
egen sum_hh=sum(nb_hh);
gen avg_density=sum_density/sum_hh;
sum avg_density;
local D=r(mean);
gen county_density=pop_density/`D';
display `D';

use "`main_dir'/Full_list", clear;	
keep if typeid==9;
keep fc_id lat lng typeid county_code;
foreach var in fc_id lat lng typeid   {;
	rename `var' `var'_sc;
	};
gen year=2018;
merge m:1 county_code year using  "`county_dir'/demo_panel_final", keepusing(pop_density);
keep if _merge==3;
drop _merge county_code;
rename pop_density density_sc; 

save "`main_dir'/SC_list", replace;
use "`main_dir'/Full_list", clear;	
keep if typeid==2 | typeid==4;
keep if cyear==2400;
keep fc_id lat lng typeid county_code;
foreach var in fc_id lat lng typeid  {;
	rename `var' `var'_fc;
	};
gen year=2018;
merge m:1 county_code year using  "`county_dir'/demo_panel_final", keepusing(pop_density);
keep if _merge==3;
drop _merge county_code;
rename pop_density density_fc ;	
cross using  "`main_dir'/SC_list";
geodist lat_sc lng_sc lat_fc lng_fc,  gen(dist) miles;	
gen congest_dist=dist*(density_fc+density_sc)/`D';
collapse (min) min_dist=dist min_congest_dist=congest_dist, by(fc_id_sc);
sum;

rename fc_id_sc fc_id;
gen dummy_connect=min_dist<=50;
*gen dummy_125=min_congest_dist<=125;
*keep dummy_125 fc_id  ;
save "`main_dir'/SC_list", replace;
list;
merge 1:1 fc_id using "`main_dir'/Full_list";
drop if dummy_connect==0;
drop dummy_connect;
drop _merge;
/* Drop facility 54: Col-located with facility 51 */
drop if fc_id==54;
drop min_dist;
save "`main_dir'/Full_list_connected", replace;

/******************************************************************************/	
/* Note: This do-file creates a list of FC-clusters by clustering togheter FC that are located within 20 miles of eachother */
/* Cluster */
foreach threshold in 0 5 10 20 30 40 50 {;
	/* Load FC list */
	use "`main_dir'/Full_list_connected", clear;	
	*local threshold2=2*`threshold';
	local threshold2=200;
	gen cluster_id=fc_id;
	forvalues j=1/1 {;
		local n=_N;
		quietly{;
			forvalues i=1/`n' {;
				gen y=lat if _n==`i';
				gen x=lng if _n==`i';
				gen state_i=state_code if _n==`i';
				gen id=cluster_id if _n==`i';
				sort id;
				foreach var in x y id state_i {;
					carryforward `var', replace;
					};
				sort cluster_id fc_id;	
				geodist lat lng y x,  gen(dist) miles;	
				replace dist=. if _n==`i';
				bysort cluster_id: egen min_dist=min(dist); /* Smallest distance between cluster_id and FC `i' */
				bysort cluster_id: egen max_dist=max(dist); /* Largest distance between cluter_id and fC `i' */
				replace cluster_id=id if min_dist<`threshold' & max_dist<`threshold2' & state_code==state_i; /* Assign all FCs in cluster_id to FC `i' if closest distance is less than threshold one and largest distance is less than threshold2 */
				drop x y id dist min_dist state_i max_dist;	
				/* Replace lat/long coordinates by cluter centroid */
				bysort cluster_id: egen mlat=mean(lat);
				bysort cluster_id: egen mlong=mean(lng);
				replace lat=mlat;
				replace lng=mlong;
				drop mlat mlong;
				};
		};
	distinct cluster_id ;
	};
		
	/* Assign a single county to each cluster based on capacity (pick county with the largest facility) */
	bysort cluster_id: egen rank_capacity=rank(sq_ft), field ;
	sort cluster_id rank_capacity;
	replace fips="" if rank_capacity>1;
	replace county_code=. if rank_capacity>1;
	replace state_code=. if rank_capacity>1;
	carryforward fips, replace;
	carryforward county_code, replace;
	carryforward state_code, replace;
		
	/* Save FC-level data */
	keep fc_id fc_description cluster_id typeid state_code fips county_code sq_ft odate oyear cyear lat lng ;
	save  "`main_dir'/Cluster_facility_list_threshold`threshold'", replace;
	
	/* Create cluster ID */
	collapse (count) fc_id, by(cluster_id);
	sort cluster_id;
	gen cluster_id_new=_n-1;
	drop fc_id;
	merge 1:m cluster_id using "`main_dir'/Cluster_facility_list_threshold`threshold'";
	drop _merge;
	drop cluster_id;
	rename cluster_id_new cluster_id;
	label drop typeid;
	save  "`main_dir'/Cluster_facility_list_threshold`threshold'", replace;	
};

/* Construct a panel of cluster activity (from entry to 2018) */
foreach threshold in 0 5 10 20 30 40 50 {;
	use "`main_dir'/Cluster_facility_list_threshold`threshold'", clear;
	replace cyear=2018 if cyear==2400;
	forvalues y=1999/2018 {;
		gen active`y'=`y'>=oyear & `y'<=cyear;
		gen sq_ft`y'=sq_ft*active`y';		
		};
	sort fc_id cluster_id;
	drop sq_ft oyear cyear;
	reshape long sq_ft active, i(fc_id) j(year);
	
	collapse (sum) sq_ft nb_fc=active (mean) cluster_lat=lat cluster_lng=lng, by(cluster_id typeid fips county_code state_code year);
	sum typeid;
	local nb_type=r(max);
	reshape wide sq_ft nb_fc, i(cluster_id year) j(typeid);	
	/* All facilities */	
	forvalues i=1/`nb_type' {;
		replace nb_fc`i'=0 if nb_fc`i'==.;
		replace sq_ft`i'=0 if sq_ft`i'==.;
		};
	gen total_fc=0;
	gen total_sq_ft=0;
	forvalues i=1/`nb_type' {;	
		replace total_fc=total_fc+nb_fc`i';
		replace total_sq_ft=total_sq_ft+sq_ft`i';
		};
	/* Regular FC (Sortable and Non Sortable) */	
	gen total_regular_fc=0;
	gen total_regular_sq_ft=0;
	foreach i in 2 4 {;	
		replace total_regular_fc=total_regular_fc+nb_fc`i';
		replace total_regular_sq_ft=total_regular_sq_ft+sq_ft`i';
		};
	
	save "`main_dir'/Panel_cluster_type_threshold`threshold'", replace;
};


/* Observed panel */
foreach threshold in 0 5 10 20 30 40 50 {;
	use "`main_dir'/Cluster_facility_list_threshold`threshold'", clear;
	rename lat cluster_lat;
	rename lng cluster_lng;
	local swapid=0;
	gen swapid=`swapid';
	keep swapid fc_id oyear cyear sq_ft typeid cluster_id county_code cluster_lat cluster_lng;		
	joinby county_code using  "`county_dir'/demo_panel_final";
	keep swapid state_code state_abbrev cluster_id cluster_lat cluster_lng county_code fc_id sq_ft oyear cyear typeid year pop_density imputed_wage county_rent_dis_imputed;	
	gen dummy_active=year>=oyear & year<=cyear;
	sort swapid cluster_id fc_id year;
	save "`swap_dir'/Facility_panel_threshold`threshold'", replace;
	};


