#delimit;
global computer "Local";
*global computer "Linstat";
if "$computer" == "Local" {;
	local intermediate_dir "~/Dropbox/Amazon_FC/Data/Processed/Intermediate";
	local swap_dir "~/Dropbox/Amazon_FC/Data/Processed/Swap_list";
	local main_dir "~/Dropbox/Amazon_FC/Data/Processed/Main";
	local county_dir "~/Dropbox/Amazon_FC/Data/County_Characteristics";
	local FC_dir "~/Dropbox/Amazon_FC/Data/FCs";
	local prediction_dir "~/Dropbox/Amazon_FC/Data/Processed/Main";
	local Amazon_report "~/Dropbox/Amazon_FC/Data/Amazon";
	local predicted_revenue "~/Dropbox/Amazon_FC/Data/Processed/Swap_list/amazon_revenue";

	};
if "$computer" == "Linstat" {;
	local intermediate_dir "/project/houde/Amazon_FC/Data/Processed/Intermediate";
	local swap_dir "/project/houde/Amazon_FC/Data/Processed/Swap_list";
	local main_dir "/project/houde/Amazon_FC/Data/Processed/Main";
	local county_dir "/project/houde/Amazon_FC/Data/County_Characteristics";
	local FC_dir "/project/houde/Amazon_FC/Data/FCs";
	};
set more off;
local threshold=20;
set type double;

/*********************************************************************/
/* Create county demographic and tax characteristics */
use "`FC_dir'/tax_dates", clear;
rename state state_abbrev;
merge 1:1 state_abbrev using "`intermediate_dir'/State_census_codes", keep(match);
drop _merge;
gen tax_year=year(TaxDateW);
replace tax_year=tax_year+1 if month(TaxDateW)>8;
keep state_code  tax_year;
merge 1:m state_code  using "`county_dir'/demo_panel_final";
keep if _merge==3;
gen amazon_tax=tax*(year>=tax_year);
tabstat amazon_tax [fw=nb_hh], by(year) stat(mean);
gen msize=avg_spending_annual_county*nb_hh;
rename imputed_wage wage;
rename county_rent_dis_imputed rent;
rename median_value house_value;
gen hhincome_100000more=hhincome_100000to124999+hhincome_125000to149999+hhincome_150000to199999+hhincome_200000ormore;
	gen sum_age=dummy_age_15_25+dummy_age_25_35+dummy_age_35_45+dummy_age_45_55+dummy_age_55_60
		+dummy_age_60_65+dummy_age_65_75+dummy_age_75_plus;
	foreach var in dummy_age_15_25 dummy_age_25_35 dummy_age_35_45 dummy_age_45_55 dummy_age_55_60 dummy_age_60_65 dummy_age_65_75 dummy_age_75_plus {;
		replace `var'=`var'/sum_age;
		};
	drop sum_age;	
	gen mean_age=dummy_age_15_25*20+dummy_age_25_35*30+dummy_age_35_45*40+dummy_age_45_55*50+dummy_age_55_60*57.5
		+dummy_age_60_65*62.5+dummy_age_65_75*70+dummy_age_75_plus*80;
	gen pop_young=(dummy_age_15_25+dummy_age_25_35)*population;		
	gen share_young=dummy_age_15_25+dummy_age_25_35;
keep county_code state_code year pop_density nb_hh msize house_value hhincome_100000more median_income mean_age share_young pop_young wage rent lon lat tax amazon_tax;
sort county_code year;
save "`main_dir'/County_demo_variables", replace;

/*********************************************************************/
/* Generate distance matrix: County to facilities */
use "`main_dir'/Cluster_facility_list_threshold20", clear;	
use "`main_dir'/Cluster_facility_list_threshold20", clear;	
collapse (mean) lat lng state_code county_code, by(cluster_id );
gen year=2018;
rename lat cluster_lat;
rename lng cluster_long;
merge m:1 county_code year using "`county_dir'/demo_panel_final", keepusing(pop_density);
keep if _merge==3;
drop _merge year;
rename pop_density cluster_density;
rename state_code cluster_state_code;
rename county_code cluster_county_code;
save "`main_dir'/Cluster_location_threshold`threshold'", replace;

/*********************************************************************/
/* Generate County to Cluster distance */
use "`county_dir'/demo_panel_final", clear;
keep if year==2018;
rename lat county_lat;
rename lon county_long;
collapse (mean) county_lat county_long pop_density nb_hh, by(county_code state_code);
egen sum_density=sum(pop_density*nb_hh);
egen sum_hh=sum(nb_hh);
gen avg_density=sum_density/sum_hh;
sum avg_density;
local D=r(mean);
gen county_density=pop_density/`D';
display `D';

cross using "`main_dir'/Cluster_location_threshold`threshold'";
replace cluster_density=cluster_density/`D';
gen congestion=(cluster_density+county_density);
sum congestion [fw=nb_hh];
sum congestion;
geodist cluster_lat cluster_long county_lat county_long, gen(dist) miles;
gen congest_dist=dist*congestion;
sort county_code cluster_id;
keep county_code cluster_id dist congest_dist;
save "`main_dir'/Distance_matrix_county_cluster", replace;


/*********************************************************************/
/* Generate distance matrix: Clusters to clusters */
use "`main_dir'/Cluster_location_threshold`threshold'", clear;
rename cluster_lat cluster_lat0;
rename cluster_long cluster_long0;
rename cluster_id cluster_id0;
rename cluster_density cluster_density0;
rename cluster_state_code cluster_state_code0;
rename cluster_county_code cluster_county_code0;
*rename fc_id fc_id0;
*rename typeid typeid0;

cross using "`main_dir'/Cluster_location_threshold`threshold'";
sort cluster_id0 cluster_id;
rename cluster_id cluster_id1;
rename cluster_lat cluster_lat1;
rename cluster_long cluster_long1;
rename cluster_density cluster_density1;
rename cluster_state_code cluster_state_code1;
rename cluster_county_code cluster_county_code1;
*rename fc_id fc_id1;
*rename typeid typeid1;
geodist cluster_lat0 cluster_long0 cluster_lat1 cluster_long1 , gen(dist) miles;

rename dist cluster_dist;
gen congestion=(cluster_density0+cluster_density1)/`D';
gen cluster_congest_dist=cluster_dist*congestion;
sort cluster_id0 cluster_id1;
save "`main_dir'/Cluster_distance_threshold`threshold'", replace;

