#delimit;
global computer "Local";
*global computer "Linstat";
if "$computer" == "Local" {;
	local intermediate_dir "~/Dropbox/Amazon_FC/Data/Processed/Intermediate";
	local swap_dir "~/Dropbox/Amazon_FC/Data/Processed/Swap_list";
	local main_dir "~/Dropbox/Amazon_FC/Data/Processed/Main";
	local county_dir "~/Dropbox/Amazon_FC/Data/County_Characteristics";
	local FC_dir "~/Dropbox/Amazon_FC/Data/FCs";
	local reis_dir "~/Dropbox/Amazon_FC/Data/Reis";
	local tables_dir "~/Dropbox/Amazon_FC/Tables_and_Figures";
	local bls_dir "~/Dropbox/Amazon_FC/Data/Census/BLS";
	local Amazon_report "~/Dropbox/Amazon_FC/Data/Amazon";	
	local prediction_dir "~/Dropbox/Amazon_FC/Data/Processed/Main";
	};
if "$computer" == "Linstat" {;
	local intermediate_dir "/project/houde/Amazon_FC/Data/Processed/Intermediate";
	local swap_dir "/project/houde/Amazon_FC/Data/Processed/Swap_list";
	local main_dir "/project/houde/Amazon_FC/Data/Processed/Main";
	local county_dir "/project/houde/Amazon_FC/Data/County_Characteristics";
	local FC_dir "/project/houde/Amazon_FC/Data/FCs";	
	local tables_dir "/project/houde/Amazon_FC/Tables_and_Figures";		
	local Amazon_report "/project/houde/Amazon_FC/Data/Amazon";	
	};
set more off;
local threshold=20;
/*
     +-----------------------------------------+
     |                      full_type   typeid |
     |-----------------------------------------|
  1. |                     FC (Fresh)        1 |
  2. |              FC (Non-Sortable)        2 |
  3. |                     FC (Prime)        3 |
  4. |                  FC (Sortable)        4 |
  5. |            Other (Call center)        5 |
     |-----------------------------------------|
  6. |                    Other (IDX)        6 |
  7. |                Other (Returns)        7 |
  8. |   Sortation (Delivery station)        8 |
  9. | Sortation (Sortation facility)        9 |
     +-----------------------------------------+
*/


/***************************************************************************************************/
use "`county_dir'/demo_panel_final", clear;
merge 1:1 year county_code using "`main_dir'/County_predicted_spending_obs_spec2", gen(merge_county_revenue) keepusing( orders);
format orders %9.6g;

/* Rename key demographic variables */
gen msize=nb_hh*avg_spending_annual_county;
gen hh=nb_hh;
gen income=mean_income;
gen density=pop_density;
keep year county_code tax msize hh orders income density lat lon;
save "`county_dir'/demo_temp", replace;

use "`swap_dir'/Facility_panel_threshold`threshold'", clear;
keep fc_id  year cluster_lat cluster_lng typeid sq_ft dummy_active;
joinby year using "`county_dir'/demo_temp";	
geodist lat lon cluster_lat cluster_lng, gen(dist) miles;

/***************************************************************************************************/
/* Compute interaction of demographic and zones */
foreach zone in 25 50 150 {;
	gen fc_zone_`zone'=dist<=`zone';
	gen w=hh*fc_zone_`zone';
	bysort fc_id year: egen sum=sum(w);
	replace w=w/sum;
	foreach var in orders msize  hh income density tax {;
		gen `var'_`zone'=`var' if fc_zone_`zone';
		replace `var'_`zone'=`var'_`zone'*w;
		};
	drop w sum;	
	};
	
/* Collapse: Calculate weighted average demographic characteristics in neighboring counties */	
collapse (sum) msize_* income_* orders_* density_* tax_*, 
	by(fc_id cluster_lat cluster_lng year);
foreach var in fc_id cluster_lat cluster_lng {;
	rename `var' `var'0;
	};

/***************************************************************************************************/
/* Compute network vaiables */
joinby year using "`swap_dir'/Facility_panel_threshold`threshold'";
keep if dummy_active==1;
gen dummy_fc=typeid==2| typeid==4;
gen dummy_sc=typeid==9;
keep if dummy_fc==1 | dummy_sc==1;
drop if fc_id==fc_id0;
geodist  cluster_lat cluster_lng cluster_lat0 cluster_lng0, gen(dist) miles;

foreach zone in 0 50 150 {;
	gen fc_zone_`zone'=dist<=`zone';
	gen nb_sc_`zone'=fc_zone_`zone'*dummy_sc;
	gen nb_fc_`zone'=fc_zone_`zone'*dummy_fc;
	gen capacity_fc_`zone'=fc_zone_`zone'*sq_ft*dummy_fc;
};

collapse (mean) msize_* income_* orders_* density_* tax_*
	(sum) nb_sc_0 nb_fc_0 capacity_fc_0 nb_sc_150 nb_sc_50 nb_fc_150 nb_fc_50 capacity_fc_150 capacity_fc_50, by(fc_id0 year);
rename nb_fc_0 nb_fc_cluster;
rename nb_sc_0 nb_sc_cluster;
rename capacity_fc_0 capacity_fc_cluster;	
rename fc_id0 fc_id;

/***************************************************************************************************/
/* Merge with facility characteristics */
merge 1:1 fc_id year using "`swap_dir'/Facility_panel_threshold`threshold'", gen(merge_facility_panel);
save "`intermediate_dir'/Facility_panel_local_characteristics", replace;

/* Add aggregate revenue data */
use "`Amazon_report'/financial_data", clear;
keep year amazon_sales;
merge 1:m year using  "`intermediate_dir'/Facility_panel_local_characteristics", generate(merge_revenue);
save "`intermediate_dir'/Facility_panel_local_characteristics", replace;

/* Load tax dates */
use "`FC_dir'/tax_dates", clear;
rename state state_abbrev;
merge 1:1 state_abbrev using "`intermediate_dir'/State_census_codes", keep(match) gen(merge_state_code);
gen tax_year=year(TaxDateW);
replace tax_year=tax_year+1 if month(TaxDateW)>8;
save "`intermediate_dir'/tax_dates_temp", replace;

/* Compute state average tax */
use "`county_dir'/demo_panel_final", clear;
collapse (mean) state_tax=tax [fw=nb_hh], by(state_code year);
merge 1:m year state_code using "`intermediate_dir'/Facility_panel_local_characteristics", gen(merge_taxrate);
keep if merge_taxrate==3;
drop merge_taxrate;
merge m:1 state_code using  "`intermediate_dir'/tax_dates_temp", gen(merge_taxdates);
keep if merge_taxdates==1 | merge_taxdates==3;
drop merge_taxdates;
gen dummy_taxable=year>=tax_year;
gen amazon_state_tax=state_tax*dummy_taxable;
save "`intermediate_dir'/Facility_panel_local_characteristics", replace;

/* Merge with employment */
merge 1:1 fc_id year using "`main_dir'/Amazon_employment_est", gen(merge_employment);
drop if merge_employment==2;

/* Keep FC, DC and SC */
local threshold=20;
keep if typeid==2 | typeid==4 | typeid==8 | typeid==9;
/* Select variables */
keep cluster_id county_code state_code fc_id sq_ft oyear cyear typeid cluster_lat cluster_lng 
	year pop_density imputed_wage county_rent_dis_imputed dummy_active name outsource 
    avg_emp dummy_mv_emp total_emp source
	msize_* income_* orders_* density_* 
	nb_sc_cluster nb_sc_50 nb_sc_150 nb_fc_cluster nb_fc_50 nb_fc_150 capacity_fc_cluster capacity_fc_50 capacity_fc_150 ; /* Facility characteristics */
save "`main_dir'/Facility_panel_characteristics_threshold`threshold'", replace;	
use "`main_dir'/Facility_panel_characteristics_threshold`threshold'", clear;
