#delimit;
global computer "Local";
*global computer "Linstat";
if "$computer" == "Local" {;
	local intermediate_dir "~/Dropbox/Amazon_FC/Data/Processed/Intermediate";
	local swap_dir "~/Dropbox/Amazon_FC/Data/Processed/Swap_list";
	local main_dir "~/Dropbox/Amazon_FC/Data/Processed/Main";
	local county_dir "~/Dropbox/Amazon_FC/Data/County_Characteristics";
	local FC_dir "~/Dropbox/Amazon_FC/Data/FCs";
	};
if "$computer" == "Linstat" {;
	local intermediate_dir "/project/houde/Amazon_FC/Data/Processed/Intermediate";
	local swap_dir "/project/houde/Amazon_FC/Data/Processed/Swap_list";
	local main_dir "/project/houde/Amazon_FC/Data/Processed/Main";
	local county_dir "/project/houde/Amazon_FC/Data/County_Characteristics";
	local FC_dir "/project/houde/Amazon_FC/Data/FCs";
	};
set more off;
local threshold=20;

/*
     +-----------------------------------------+
     |                      full_type   typeid |
     |-----------------------------------------|
  1. |                     FC (Fresh)        1 |
  2. |              FC (Non-Sortable)        2 |
  3. |                     FC (Prime)        3 |
  4. |                  FC (Sortable)        4 |
  5. |            Other (Call center)        5 |
     |-----------------------------------------|
  6. |                    Other (IDX)        6 |
  7. |                Other (Returns)        7 |
  8. |   Sortation (Delivery station)        8 |
  9. | Sortation (Sortation facility)        9 |
     +-----------------------------------------+
*/


/* Note: This file computes a list of swaps for a given cluster threshold value, along with swap characteristics */
/**************************************************************************/
/* Create the universe of possible swaps */
foreach i in A B {;
	use "`main_dir'/Cluster_facility_list_threshold`threshold'", clear;
	sum;
	/* Define count of number of types (used in loops below) */
	tab typeid, nolabel;
	sum typeid;
	local nb_type=r(max);
	/* Relabel variables: A/B */
	rename fc_id fc_id`i';
	rename cluster_id cluster_id`i';
	rename lat cluster_lat`i';
	rename lng cluster_lng`i';
	rename sq_ft sq_ft`i';
	rename oyear oyear`i';
	rename cyear cyear`i';
	rename odate odate`i';
	rename typeid typeid`i';
	rename fips fips`i';
	rename state_code state_code`i';
	rename county_code county_code`i';	
	keep fc_id`i' cluster_id`i' cluster_lat`i' cluster_lng`i' cyear`i' oyear`i' odate`i' sq_ft`i'  typeid`i' county_code`i' fips`i' state_code`i';
	save "`intermediate_dir'/Cluster_list_threshold`threshold'_temp`i'", replace;
	};
use "`intermediate_dir'/Cluster_list_threshold`threshold'_tempA", clear;

/* Cross all investment events */
cross using "`intermediate_dir'/Cluster_list_threshold`threshold'_tempB";
/* Drop duplicates */
drop if fc_idA==fc_idB;
/* Drop investments that lead to a closure */
drop if cyearA<2040;
/* Keep swaps that are more than 1 years appart */
keep if oyearB-oyearA>1;
/* Restrict swaps to be of the same type (merging the two FC types) */
/* Temporarily replace regular FC types */
gen typeidA_temp=typeidA;
gen typeidB_temp=typeidB;
replace typeidA_temp=2 if typeidA==4;
replace typeidB_temp=2 if typeidB==4;
drop if typeidA_temp!=typeidB_temp;
drop typeidA_temp typeidB_temp;
/* Drop swaps involving IDX and return centers */
drop if typeidA==6 | typeidA==7;
drop if typeidB==6 | typeidB==7;
gen swapid=_n;
local nb_swap=_N;
*local nb_swap=10;
display "nb swap: " `nb_swap';

/* Characteristics of the swaps */
geodist cluster_latA cluster_lngA cluster_latB cluster_lngB,  gen(swap_dist) miles;
gen diff_year=oyearB-oyearA;
gen indicator_same_state=state_codeA==state_codeB;
gen indicator_same_cluster=cluster_idA==cluster_idB;
gen diff_size=sq_ftB-sq_ftA;
save "`swap_dir'/Swap_list", replace;
