#delimit;
global computer "Local";
*global computer "Linstat";
if "$computer" == "Local" {;
	local intermediate_dir "~/Dropbox/Amazon_FC/Data/Processed/Intermediate";
	local swap_dir "~/Dropbox/Amazon_FC/Data/Processed/Swap_list";
	local main_dir "~/Dropbox/Amazon_FC/Data/Processed/Main";
	local county_dir "~/Dropbox/Amazon_FC/Data/County_Characteristics";
	local FC_dir "~/Dropbox/Amazon_FC/Data/FCs";
	};
if "$computer" == "Linstat" {;
	local intermediate_dir "/project/houde/Amazon_FC/Data/Processed/Intermediate";
	local swap_dir "/project/houde/Amazon_FC/Data/Processed/Swap_list";
	local main_dir "/project/houde/Amazon_FC/Data/Processed/Main";
	local county_dir "/project/houde/Amazon_FC/Data/County_Characteristics";
	local FC_dir "/project/houde/Amazon_FC/Data/FCs";
	};
set more off;
local threshold=20;

/*
     +-----------------------------------------+
     |                      full_type   typeid |
     |-----------------------------------------|
  1. |                     FC (Fresh)        1 |
  2. |              FC (Non-Sortable)        2 |
  3. |                     FC (Prime)        3 |
  4. |                  FC (Sortable)        4 |
  5. |            Other (Call center)        5 |
     |-----------------------------------------|
  6. |                    Other (IDX)        6 |
  7. |                Other (Returns)        7 |
  8. |   Sortation (Delivery station)        8 |
  9. | Sortation (Sortation facility)        9 |
     +-----------------------------------------+
*/

/* Counter-factual panel: Facility panel */
use "`swap_dir'/Swap_list", clear;
sum swapid;
local nb_swap=r(max);

use "`main_dir'/Cluster_facility_list_threshold`threshold'", clear;
distinct cluster_id;;
rename lat cluster_lat;
rename lng cluster_lng;
local swapid=0;
gen swapid=`swapid';
keep swapid fc_id oyear cyear sq_ft typeid cluster_id county_code cluster_lat cluster_lng;		
joinby county_code using  "`county_dir'/demo_panel_final";
keep swapid state_code state_abbrev cluster_id cluster_lat cluster_lng county_code 
	fc_id sq_ft oyear cyear typeid year pop_density imputed_wage county_rent_dis_imputed;	
gen dummy_active=year>=oyear & year<=cyear;
sort swapid cluster_id fc_id year;
save "`swap_dir'/Facility_panel_swap`swapid'", replace;
distinct cluster_id;

forvalues swapid=1/`nb_swap' {;
	display "swap " `s';
	use "`main_dir'/Cluster_facility_list_threshold`threshold'", clear;
	rename lat cluster_lat;
	rename lng cluster_lng;
	gen swapid=`swapid';
	merge m:1 swapid using "`swap_dir'/Swap_list", keep(match);	
	/* Swap opening dates */
	replace oyear=oyearB if fc_id==fc_idA;
	replace oyear=oyearA if fc_id==fc_idB;	
	/* Swap capacity: Within cluster only */
	replace sq_ft=sq_ftB if fc_id==fc_idA & cluster_idA==cluster_idB;
	replace sq_ft=sq_ftA if fc_id==fc_idB & cluster_idA==cluster_idB;
	keep swapid fc_id oyear cyear sq_ft typeid cluster_id county_code cluster_lat cluster_lng;		
	joinby county_code using  "`county_dir'/demo_panel_final";
	keep swapid state_code state_abbrev cluster_id cluster_lat cluster_lng county_code 
		fc_id sq_ft oyear cyear typeid year pop_density imputed_wage county_rent_dis_imputed;	
	gen dummy_active=year>=oyear & year<=cyear;
	sort swapid cluster_id fc_id year;
	save "`swap_dir'/Facility_panel_swap`swapid'", replace;
	distinct cluster_id;
	};

/**************************************************************************/
/* Construct physical presence: Date of first entry (FC and other) for each swap */
forvalues s=0/`nb_swap' {;
	use "`intermediate_dir'/State_census_codes", clear;
	merge 1:m state_code using "`swap_dir'/Facility_panel_swap`s'", keep(match);	
	tab _merge;
	keep if dummy_active==1;
	gen year_fc=year if typeid==2 | typeid==4;
	collapse (min) first_presence_state=year first_fc_state=year_fc, 
		by(state_code census_region census_division state state_abbrev state_code_string);
	/* Calculate first FC entry year (using network from swap) */	
	sum first_fc_stat first_presence_state;
	replace first_fc_state =2050 if first_fc_state  ==.;
	gen swapid=`s';
	save "`swap_dir'/State_physical_presence_swap`s'", replace;
	};	
