#include<oxstd.h>
#import <maximize>
#import <solvenle>
#include <oxdraw.h>
#include <oxfloat.h>
#include <oxprob.h>

enum{cross_county_code,cross_cluster_id,cross_dist,cross_congest_dist}; /* Cross-section: County-cluster list */

enum{fc_year,fc_state_code,fc_id,fc_msize_25,fc_msize_50,fc_msize_150,fc_income_25,fc_income_50,fc_income_150,fc_orders_25,fc_orders_50,fc_orders_150,
     fc_density_25,fc_density_50,fc_density_150,fc_nb_sc_cluster,fc_nb_fc_cluster,fc_capacity_fc_cluster,fc_nb_sc_150,fc_nb_sc_50,fc_nb_fc_150,
     fc_nb_fc_50,fc_capacity_fc_150,fc_capacity_fc_50,fc_cluster_id,fc_county_code,fc_sq_ft,fc_oyear,fc_cyear,fc_typeid,fc_cluster_lat,
     fc_cluster_lng,fc_pop_density,fc_county_rent_dis_imputed,fc_imputed_wage,fc_dummy_active,fc_total_emp,fc_name,fc_outsource,
     fc_source,fc_avg_emp,fc_dummy_mv_emp}; /* Facility characteristics */

enum{type_fresh=1,type_nonsortable,type_prime,type_sortable,type_callcenter,type_idx,type_returns,type_delivery,type_sortation};

enum{cluster_id0,cluster_lat0,cluster_long0,cluster_state_code0,cluster_county_code0,cluster_density0,cluster_id1,
     cluster_lat1,cluster_long1,cluster_state_code1,cluster_county_code1,cluster_density1,
     cluster_dist,congestion,cluster_congest_dist}; /* Cluster distance */

enum{demo_state_code,demo_year,demo_pop_density,demo_nb_hh,demo_median_income,demo_house_value,
     demo_county_code,demo_rent,demo_wage,demo_lon,demo_lat,demo_tax,demo_amazon_tax,demo_msize,
     demo_hhincome_100000more,demo_mean_age,demo_pop_young,demo_share_young}; /* County-Demographic file */

enum{demand_county_code,demand_year,demand_orders,demand_net_revenue,demand_spending}; /* Demand and revenue data */
enum{price_year,price_avg,price_iqr};


/*
  1. |                     FC (Fresh)        1 |
  2. |              FC (Non-Sortable)        2 |
  3. |                     FC (Prime)        3 |
  4. |                  FC (Sortable)        4 |
  5. |            Other (Call center)        5 |
  |-----------------------------------------|
  6. |                    Other (IDX)        6 |
  7. |                Other (Returns)        7 |
  8. |   Sortation (Delivery station)        8 |
  9. | Sortation (Sortation facility)        9 |
*/

#import<logistic_func>
#import<estimation_func>

decl iprint=1;

/* Common variables */
enum{imu_0,imu_k,imu_t,ialpha,iAfc,iAsc,ibeta,igamma,nbParam};
enum{iNetwork_sortation,iSortation_zone,iDist_fc_sc};
decl aParamName={"$\\mu_0$","$\\mu_k$","$\\mu_t$","$\\alpha$","$A_{fc}$","$A_{sc}$","$\\beta$","$\\gamma$"};
decl vCluster_id,vCounty_id,vYear,vUnique_fcid,vVariety;
decl N,M,T,nbFC;
decl mDummy_cluster,mDist_county_cluster,mFC_activity,mCounty_demand,mCounty_demand_pred,mCounty_hh,vFC_capacity;
decl mCounty_high_income,mCounty_age,mCounty_share_young,mCounty_pop_young;
decl vFC_avg_emp,vOutsource,vTotal_emp,mDistance_cluster,vFC_type,iMV;
decl vLogEmpl_obs,mID_empl,mCharact,vBeta,vSE_total,vSE_aux,mCharact_trend,vTrend_coef,aCoef_name,mWeight,vMoment_id;
decl vAvgLabor_pred,vTotal_emp_pred;
decl aID,aNetwork_combined;
decl vParam_logistic,vOPTid;
decl iVariety=1;
decl iExogenous=0;
decl iSortThreshold,iFCThreshold;
decl iSC_network_connect=1;
decl scale;
decl iMinDist;
decl iNearestFC;

matching_estimation(const iSpec,const iEstimator)
{
  format(1000);
  decl i,j,k;
  /**********************************************************/
  /* Parameters */
  decl mu_0=0.41; /* If mu_0 is close to one, sorting is based on distance alone. When mu_0 is close to zero, sorting is based on FC size. */
  decl mu_k=0.69; /* If mu_k=1, two FCs of the same size is equivalent to one large FC. 
		     If mu_k<1, it is more productive to use one large than two smaller. 
		     If mu_k>1, it is more productive to use two small FCs. */
  decl Afc=0.5; /* TFP parameter: FC */
  decl Asc=0.15; /* TFP parameter: SC */
  decl alpha=-1;
  decl beta=1;
  
  /**********************************************************/
  /* Load Data-sets */
  decl main_dir="../../Data/Processed/Main/";
  decl output_dir="../output/";
  decl swap_dir="../../Data/Processed/Swap_list/";
  scale=1000000; /* Capacity scale variable */
  decl aVarName_cross,aVarName_fc,aVarName_demand,aVarName_distance,aVarName_demo;
  decl mCrossData=loadmat(sprint(main_dir,"Distance_matrix_county_cluster.dta"),&aVarName_cross);
  decl mFacilityData=loadmat(sprint(main_dir,"Facility_panel_characteristics_threshold20.dta"),&aVarName_fc);
  decl mDistanceData=loadmat(sprint(main_dir,"Cluster_distance_threshold20.dta"),&aVarName_distance);
  mFacilityData[][fc_sq_ft]./=scale;
  decl mDemandData=loadmat(sprint(main_dir,"County_predicted_spending_obs_spec",iSpec,".dta"),&aVarName_demand);
  mDemandData=sortbyc(mDemandData,<demand_county_code,demand_year>);
  decl aDemoVar_names;
  decl mDemoData=loadmat(sprint(main_dir,"County_demo_variables.dta"),&aDemoVar_names);
  mDemoData=sortbyc(mDemoData,<demo_county_code,demo_year>);  
  println("%r",aDemoVar_names,(meanc(mDemoData)|sqrt(varc(mDemoData)))');
  println("%r",aVarName_demand,(meanc(mDemandData)|sqrt(varc(mDemandData)))');
  decl aVarName_variety;
  decl mVariety_index=loadmat(sprint(main_dir,"Variety_index_spec",iSpec,".dta"),&aVarName_variety);
  println("%c",aVarName_variety,mVariety_index);
  vVariety=mVariety_index[][1];
  
  /* Sort cross-data by county and cluster ID */
  mCrossData=sortbyc(mCrossData,<cross_county_code,cross_cluster_id>);
  mFacilityData=sortbyc(mFacilityData,<fc_id,fc_year>); 
  mDistanceData=sortbyc(mDistanceData,<cluster_id0,cluster_id1>);
  println("variable names (panel): ",aVarName_fc);
  println("unique type: ",unique(mFacilityData[][fc_typeid]));
  
  /* Vector of IDs */
  vCluster_id=unique(mCrossData[][cross_cluster_id]);
  vCounty_id=unique(mCrossData[][cross_county_code]);
  vUnique_fcid=unique(mFacilityData[][fc_id]);
  vYear=unique(mFacilityData[][fc_year]);
  N=columns(vCluster_id);
  M=columns(vCounty_id);
  T=columns(vYear);
  nbFC=columns(vUnique_fcid);

  /* Reshape wide facility data: Activity and cost */
  decl mFC_id=reshape(mFacilityData[][fc_id],nbFC,T);
  mFC_activity=reshape(mFacilityData[][fc_dummy_active],nbFC,T);
  vFC_capacity=reshape(mFacilityData[][fc_sq_ft],nbFC,T)[][0];
  vFC_avg_emp=reshape(mFacilityData[][fc_avg_emp],nbFC,T)[][0];
  vOutsource=reshape(mFacilityData[][fc_outsource],nbFC,T)[][0];
  vTotal_emp=reshape(mFacilityData[][fc_total_emp],nbFC,T)[0][]/1000;
  vFC_type=reshape(mFacilityData[][fc_typeid],nbFC,T)[][0];
  decl vFC_cyear=reshape(mFacilityData[][fc_cyear],nbFC,T)[][0];
  decl vFC_oyear=reshape(mFacilityData[][fc_oyear],nbFC,T)[][0];
  decl vFC_cluster=reshape(mFacilityData[][fc_cluster_id],nbFC,T)[][0];
  println("vTotal-emp: ",vTotal_emp);
  println("nb of active facilities per year: ",sumc(mFC_activity));

  /* Reshape wide distance matrix */
  println("dim distance matrix: ",rows(mDistanceData)~columns(mDistanceData));
  println("cluster_dist: ",cluster_dist);
  mDist_county_cluster=reshape(mCrossData[][cross_dist],M,N);
  mDistance_cluster=reshape(mDistanceData[][cluster_dist],N,N);

  /* Reshape wide demand data */
  //decl vPackage_size=range(1,5,4/(T-1));
  //decl vPackage_size=constant(4,1,T);
  decl vPackage_size=constant(1,1,T);
  println("pacakge size: ",vPackage_size);

  println("T: ",T);
  println("dim package size: ",rows(vPackage_size)~columns(vPackage_size));

  mCounty_demand=(reshape(mDemandData[][demand_orders],M,T)/scale)./vPackage_size;
  mCounty_hh=reshape(mDemoData[][demo_nb_hh],M,T);
  mCounty_high_income=reshape(mDemoData[][demo_hhincome_100000more],M,T).*mCounty_hh;
  mCounty_age=reshape(mDemoData[][demo_mean_age],M,T);
  mCounty_share_young=reshape(mDemoData[][demo_share_young],M,T);
  mCounty_pop_young=reshape(mDemoData[][demo_pop_young],M,T);
  decl vAvgOrder_hh=meanc(mCounty_demand./(mCounty_hh/scale));
  println("avg orders per hh: ",(vYear|vAvgOrder_hh)');
  
  /* Indicator function classifying each regular fulfillment center in each cluster: (nb facility) x N) */
  mDummy_cluster=(vFC_cluster.==vCluster_id); /* nbFC x Nb cluster */
  decl vDummy_sc=vFC_type.==type_sortation;
  decl vDummy_fc=(vFC_type.==type_sortable)+(vFC_type.==type_nonsortable);
  decl id_fc=vecindex(vDummy_fc,1);
  decl id_sc=vecindex(vDummy_sc,1);
  aID=new array[2];
  aID[0]=id_fc;
  aID[1]=id_sc;
  println("nb of each types: ","%c",{"fc","sc"},rows(id_fc)~rows(id_sc));

  /********************************************************************************************/
  /* Chracteristics: Auxiliary regression */
  decl mOrders_25=reshape(mFacilityData[][fc_orders_25],nbFC,T);
  decl mOrders_50=reshape(mFacilityData[][fc_orders_50],nbFC,T);
  decl mOrders_150=reshape(mFacilityData[][fc_orders_150],nbFC,T);
  decl mPopulation_25=reshape(mFacilityData[][fc_msize_25],nbFC,T);
  decl mPopulation_50=reshape(mFacilityData[][fc_msize_50],nbFC,T);
  decl mPopulation_150=reshape(mFacilityData[][fc_msize_150],nbFC,T);
  decl mPopDensity_25=reshape(mFacilityData[][fc_density_25],nbFC,T);
  decl mPopDensity_50=reshape(mFacilityData[][fc_density_50],nbFC,T);
  decl mPopDensity_150=reshape(mFacilityData[][fc_density_150],nbFC,T);
  decl mNb_fc_cluster=reshape(mFacilityData[][fc_nb_fc_cluster],nbFC,T);
  decl mCap_fc_cluster=reshape(mFacilityData[][fc_capacity_fc_cluster],nbFC,T);
  decl mIndicator_fc=(mNb_fc_cluster.>0);
  decl vLogCapacity=log(vFC_capacity);
  decl mLogPopulation_25=log(mPopulation_25);
  decl mLogPopulation_50=log(mPopulation_50);
  decl mLogPopulation_150=log(mPopulation_150);
  decl mLogPopdensity_25=log(mPopDensity_25);
  decl mLogPopdensity_50=log(mPopDensity_50);
  decl mLogPopdensity_150=log(mPopDensity_150);
  decl mLogOrders_25=log(mOrders_25);
  decl mLogOrders_50=log(mOrders_50);
  decl mLogOrders_150=log(mOrders_150);

  /* Create column ID for each plant with positive employment */
  mID_empl=selectifr(id_fc|id_sc,vFC_avg_emp[id_fc|id_sc].>0);
  mID_empl=deleteifr(mID_empl,vFC_cyear[mID_empl].<2018);
  mID_empl=deleteifr(mID_empl,vFC_oyear[mID_empl].==2018);
  mID_empl~=zeros(rows(mID_empl),1);
  println("dimension mID_empl: ",rows(mID_empl)~columns(mID_empl));
  mCharact=new matrix[rows(mID_empl)][4];
  vLogEmpl_obs=new matrix[rows(mID_empl)][1];
  decl t;
  for(i=0;i<rows(mID_empl);i++)
    {
      t=maxc(vecindex(mFC_activity[mID_empl[i][0]][],1));
      mID_empl[i][1]=t;
      mCharact[i][]=1~(((mLogPopdensity_50[][t]).*vDummy_fc)~(vDummy_fc.*vLogCapacity)~vDummy_sc)[mID_empl[i][0]][];
      vLogEmpl_obs[i]=log(vFC_avg_emp[mID_empl[i][0]]);
    }

  /* Auxiliary regression */
  println("mean facility characteristics: ",meanc(mCharact));
  aCoef_name={"Intercept","Log density 50","Log capacity - FC","1(Type=SC)","SC X Log Nb FC in 50 Miles"};
  olsc(vLogEmpl_obs,mCharact,&vBeta);
  decl s2=sumsqrc(vLogEmpl_obs-mCharact*vBeta)/(rows(vLogEmpl_obs)-columns(mCharact));
  olsc(vLogEmpl_obs,mCharact,&vBeta);
  decl vResid=vLogEmpl_obs-mCharact*vBeta;
  decl mV_ols=invert(mCharact'mCharact)*((mCharact.*(vResid.^2))'mCharact)*invert(mCharact'mCharact);
  //decl mV_ols=invert(mCharact'mCharact)*s2;
  decl vSE_ols=sqrt(diagonal(mV_ols)');
  vSE_total=(sqrt(sumc(mFC_activity))').^(-1);
  println("vSE_ols: ",vSE_ols);
  println("vSE_total: ",vSE_total);
  println("Auxiliary regression parameters (t-stat): ","%r",aCoef_name,vBeta~fabs(vBeta./vSE_ols));    
  if(iprint) println("distribution of capacity for regular FCs: ",quantilec(vFC_capacity[id_fc],<0,1/4,1/2,3/4,1>)');
  if(iprint) println("number of regular FC: ",rows(id_fc));

  /* Auxilary regression: Aggregrate trend */
  mCharact_trend=(range(1,T-1));
  mCharact_trend-=meanr(mCharact_trend);
  decl vY=log(vTotal_emp[:T-2])-meanr(log(vTotal_emp[:T-2]));
  olsr(vY,mCharact_trend,&vTrend_coef);
  println("vTrend_coef: ",vTrend_coef);
  s2=sumsqrr(vY-vTrend_coef*mCharact_trend)/(T-3-rows(mCharact_trend));
  println("s2: ",s2);
  decl vSE_trend=sqrt(diagonal(invert(mCharact_trend*mCharact_trend')*s2));
  decl mV_trend=invert(mCharact_trend*mCharact_trend')*s2;
  println("se trend: ",vSE_trend|(vTrend_coef./vSE_trend));
  println((vTrend_coef|vSE_trend|fabs(vTrend_coef./vSE_trend))');

  decl vAux_moments=vBeta|vTrend_coef|log(vTotal_emp[T-2]);
  mWeight=new matrix[rows(vAux_moments)][rows(vAux_moments)];
  mWeight[:rows(vBeta)-1][:rows(vBeta)-1]=invert(mV_ols);
  mWeight[rows(vBeta)][rows(vBeta)]=invert(mV_trend);
  mWeight[rows(vBeta)+1][rows(vBeta)+1]=(0.0001)^(-1);
  println("mWeight: ",mWeight);
  vMoment_id=range(0,rows(vAux_moments)-1);
  println("mWeight: ",mWeight[vMoment_id][vMoment_id]);  
  iMV=-9999;
  
  /********************************************************************************************/
  /* Create year-by-year sortation and delivery network */
  iSortThreshold=150;
  iFCThreshold=25;
  sortation_network_serial(&aNetwork_combined,mDummy_cluster);
   
  /********************************************************/
  /* Estimation */
  decl vP,Q;
  decl vResid_coef;
  decl mJacobian;
  decl SSR;
  decl mV,vSE_nl,vSE_param;
  decl mOutput_pred,mLabor_pred,mDist_pred,mNetwork_dist,vTotal_emp_pred;        
  
  /* Starting values and parameter space */
  vParam_logistic=new matrix[nbParam][1];
  vParam_logistic=loadmat(sprint("Results/matching_function_param_",iEstimator,"_starting.mat"));  
  if(iNearestFC==0) vOPTid=<imu_0;iAfc;iAsc;ialpha>;
  if(iNearestFC==1)
    {
      vOPTid=<iAfc;iAsc;ialpha>;
      vParam_logistic[imu_0]=100;
    }
  decl mHH_dist,vHH_vi,vHH_dist,iHH_vi;
  
  if(iEstimator=="NLS") {
    /* NLS */
    iprint=1;
    vP=vParam_logistic[vOPTid];
    vParam_logistic[imu_t]=0;
    println("vParam_logistic : ",vParam_logistic);
    nls_func(vP,&SSR,0,0);
    println("SSR: ",SSR);
    
    iprint=0;
    MaxControl(100,1);
    MaxBFGS(nls_func,&vP,&Q,0,1);
    iprint=0;
    resid_func(&vResid_coef,vP);
    NumJacobian(resid_func,vP,&mJacobian);
    mV=varc(vResid_coef)*invert(mJacobian'mJacobian);
    vParam_logistic[vOPTid]=vP;
    vSE_nl=sqrt(diagonal(mV))';
    vSE_param=new matrix[nbParam][1];    
    vSE_param[vOPTid]=vSE_nl;

    /* Evaluate the moment function at the NLS estimate */
    iprint=1;
    moment_func(&vResid_coef,vP);
    if(iNearestFC==0) {
      savemat("Results/matching_function_stockout_resid_nls.csv",vResid_coef);
      savemat(sprint("Results/auxiliary_regression_moments_stockout_spec",iSpec,".csv"),(vBeta|vTrend_coef|log(vTotal_emp[T-2]))~invert(mWeight));
    }
    if(iNearestFC==1) {
      savemat("Results/matching_function_nearestfc_resid_nls.csv",vResid_coef);
      savemat(sprint("Results/auxiliary_regression_moments_nearestfc_spec",iSpec,".csv"),(vBeta|vTrend_coef|log(vTotal_emp[T-2]))~invert(mWeight));
    }
    output_func_mp(&mOutput_pred,&mLabor_pred,&mDist_pred,&mNetwork_dist,aNetwork_combined,mDummy_cluster,mCounty_demand);
    vTotal_emp_pred=sumc(mLabor_pred)/1000; 
    println("Employment: ",vTotal_emp_pred'~vTotal_emp');  
    iprint=0;
  }
  
  if(iEstimator=="GMM") {
    /* GMM */
    println("//////////////////////////////////");
    println("iSortThreshold: ",iSortThreshold);
    println("iVariety: ",iVariety);
    iprint=1;
    vParam_logistic[imu_0]=1;
    vParam_logistic[imu_t]=0;
    vP=vParam_logistic[vOPTid];	
    moment_func(&vResid_coef,vP);

    iprint=0;
    MaxControl(100,1);
    MaxBFGS(gmm_func,&vP,&Q,0,1);
    vParam_logistic[vOPTid]=vP;
    NumJacobian(moment_func,vP,&mJacobian);
    mV=invert(mJacobian'mWeight*mJacobian);
    vSE_nl=sqrt(diagonal(mV))';
    vSE_param=new matrix[nbParam][1];
    vSE_param[vOPTid]=vSE_nl;
    
    println("nb Param: ",nbParam);
    println("rows(vSE_param): ",rows(vSE_param));
    println("rows(vParam_logistic): ",rows(vParam_logistic));
    if(iNearestFC==1) {
      savemat(sprint("Results/matching_function_nearestfc_resid_spec",iSpec,".csv"),vResid_coef);
      savemat(sprint("Results/matching_function_nearestfc_obj_spec",iSpec,".csv"),Q);
      savemat(sprint("Results/auxiliary_regression_moments_nearestfc_spec",iSpec,".csv"),(vBeta|vTrend_coef|log(vTotal_emp[T-2]))~invert(mWeight));
    }
    if(iNearestFC==0) {
      savemat(sprint("Results/matching_function_stockout_resid_spec",iSpec,".csv"),vResid_coef);
      savemat(sprint("Results/matching_function_stockout_obj_spec",iSpec,".csv"),Q);
      savemat(sprint("Results/auxiliary_regression_moments_stockout_spec",iSpec,".csv"),(vBeta|vTrend_coef|log(vTotal_emp[T-2]))~invert(mWeight));
    }    
  }


  /* Compute distance from logistic model */
  decl vParam_logistic_true=vParam_logistic;
  vParam_logistic[imu_0]=1;
  mHH_dist=new matrix[T][2];
  vHH_vi=new matrix[T][1];
  for(t=0;t<T;t++)
    {
      logistic_hh_coverage(&vHH_dist,&iHH_vi,aNetwork_combined,mDummy_cluster,t);
      mHH_dist[t][]=vHH_dist;
      vHH_vi[t]=iHH_vi;
    }
  vParam_logistic[imu_0]=vParam_logistic_true[imu_0];
  println("HH distance and VI : ",mHH_dist~vHH_vi);
  println("nb Param: ",nbParam);
  println("Estimation results: ","%r",aParamName,"%c",{"coef","se"},vParam_logistic~vSE_param);

  if(iNearestFC==0) {
    savemat(sprint("Results/matching_function_stockout_param_",iEstimator,"_variety",iVariety,"_spec",iSpec,".mat"),vParam_logistic);
    savemat(sprint("Results/matching_function_stockout_param_",iEstimator,"_variety",iVariety,"_spec",iSpec,".csv"),vParam_logistic);
    savemat(sprint("Results/matching_function_stockout_param_se_",iEstimator,"_variety",iVariety,"_spec",iSpec,".csv"),vSE_param);
  }
  if(iNearestFC==1) {
    savemat(sprint("Results/matching_function_nearestfc_param_",iEstimator,"_variety",iVariety,"_spec",iSpec,".mat"),vParam_logistic);
    savemat(sprint("Results/matching_function_nearestfc_param_",iEstimator,"_variety",iVariety,"_spec",iSpec,".csv"),vParam_logistic);
    savemat(sprint("Results/matching_function_nearestfc_param_se_",iEstimator,"_variety",iVariety,"_spec",iSpec,".csv"),vSE_param);
  }

  savemat("Results/sortation_thresholds.csv",iFCThreshold|iSortThreshold);
  savemat(sprint(main_dir,"Network_density_year_",iEstimator,"_variety",iVariety,"_spec",iSpec,".csv"),vYear'~mHH_dist~vHH_vi,{"year","hh_dist","hh_dist_vi","hh_vi"});
}

main()
{
  /*********MODEL CHANGES DEPENDING ON HOW WE TREAT THE REGION EFFECTS.**********/ 
  /**********MODEL=0 NO REGION EFFECTS********************************************/
  /*********MODEL=1 REGION EFFECTS***********************************************/
  /*********MODEL=2 DISTRICT EFFECTS*********************************************/
  /*********MODEL=3 REGION AMAZON EFFECTS****************************************/
  /*********MODEL=4 DISTRICT AMAZON EFFECTS**************************************/
  /*********MODEL=5 REGION AMAZON EFFECTS 2015-2016******************************/
  /*********MODEL=6 DISTRICT AMAZON EFFECTS  2015-2016***************************/
  
  decl i;
  decl iEstimator="NLS";
  iVariety=1;
  iNearestFC=1;
  matching_estimation(2,iEstimator);
  iNearestFC=0;
  /*
  for(i=0;i<=6;i++)
    {
      println("///////////////////////////////////////////");
      println("CES model specification: ",i);
      matching_estimation(i,iEstimator);
    }
  iNearestFC=1;
  for(i=0;i<=6;i++)
    {
      println("///////////////////////////////////////////");
      println("CES model specification: ",i);
      matching_estimation(i,iEstimator);
    }
  */
}