#include<oxstd.h>
#import <maximize>
#import <solvenle>
#include <oxdraw.h>
#include <oxfloat.h>
#include <oxprob.h>
#import <maxsqp>

#import<program_files/network_logistic_batch>
#import<program_files/annual_spending_func>
#import<program_files/enumerate>

enum{cross_county_code,cross_cluster_id,cross_dist}; /* Cross-section: County-cluster list */
enum{cluster_id0,state_code0,cluster_id1,state_code1,cluster_dist}; /* Cluster distance */
enum{type_fresh=1,type_nonsortable,type_prime,type_sortable,type_callcenter,type_idx,type_returns,type_delivery,type_sortation};

enum{demo_state_code,demo_year,demo_pop_density,demo_nb_hh,demo_median_income,demo_house_value,
     demo_county_code,demo_rent,demo_wage,demo_lon,demo_lat,demo_tax,demo_amazon_tax,demo_msize,
     demo_hhincome_100000more,demo_mean_age,demo_pop_young,demo_share_young}; /* County-Demographic file */

enum{demand_year,demand_county,demand_mode,demand_nb_hh,demand_avg_spending_annual_county,
     demand_alpha,demand_tax_rate,demand_tax_status,demand_density1,demand_perc_100,demand_tax_coeff,
     demand_tax_coeff100,demand_density_coeff,demand_am_share,demand_margin0,demand_prods_per,
     demand_pre_laprice1,demand_pre_lmprice1,demand_pre_liquart1,demand_pre_lstd1,demand_a_margin,demand_mp_margin}; /* Demand variables */

enum{cluster_county_code,cluster_lat,cluster_long,cluster_id,cluster_state_code,cluster_year,
     cluster_pop_density,cluster_county_rent_dis_imputed,cluster_imputed_wage}; /* Cluster_characteristics */


/* Common variables */
enum{imu_0,imu_k,imu_t,ialpha,iAfc,iAsc,ibeta,igamma,nbParam};
enum{iShip_int,iShip_dist,iShip_vi,iFC_dens};
enum{iNetwork_sortation,iSortation_zone,iDist_fc_sc};
decl aParamName={"$\\mu_0$","$\\mu_k$","$\\mu_t$","$\\alpha$","$A_{fc}$","$A_{sc}$","$\\beta$","$\\gamma$"};
decl vYear,vVariety;
decl N,M,T,nbFC;
decl mDist_county_cluster,mDummy_in_network,mInvDist_county_cluster;
decl mDistance_cluster;
decl vParam_logistic;
decl vCluster_id,vCounty_id;
decl mDummy_cluster_county,vTax_delay,mTotalSpending,mCES_param,iMu_param,iMu_amazon,iMu_mp,aSpendingInt,scale,vState_code;
decl vAmazonShare,mAmazonPrice_tax_excl,mRetailSpending;
decl vNearestCounty_id,mCounty_wage,mCounty_rent,mCounty_density,mCounty_tax,mCounty_hh,mCluster_wage,mCluster_density,mCluster_rent,vParam_cost;
decl mCounty_high_income,mCounty_age,mCounty_share_young,mCounty_pop_young,vNetwork_density_obs;
decl iSortThreshold=150;
decl iFCThreshold=25;
decl iMinDist=25;
decl iConvenience;
decl iprint=1;
decl iFC_cap=1;
decl iSC_cap=0.3;
decl iSingleFC=0;
decl iNearestFC=0;
decl iDummy_SC;
decl year;
decl iSpec_label;

/************************************************************/
/* Specifications */
decl iTaxDelay,iVariety,iDensityParam,iLargeCES,iEstimator,iClusterThreshold;
enum{iLR_fc,iLR_sc,iMUT_PROB,iMUT_SHIFT,iMAXIT,iSequential_update,iSim,inbRuns,iGreedy,nbTuning_param};
decl aTuning_param={"LR_fc","LR_sc","MUT_PROB","MUT_SHIFT","MAXIT","iSequential_update","Sim","nbRuns","Greedy"};
/*************************************************************/

network_solution(const iSpec,const iSC)
{
  format(1000);
  decl i,j,k,id;

  /************************************************************************/
  /* Batch specifiations */
  iDummy_SC=iSC;
  decl mSpecification_list;
  mSpecification_list=loadmat("data_files/Specification_list.mat");
  iSpec_label=iSpec;
  decl iSolutionYear=mSpecification_list[iSpec][0];
  decl iSolutionTax=mSpecification_list[iSpec][1];
  decl iSolutionSeed=mSpecification_list[iSpec][2];
  println("Specification details: ",mSpecification_list[iSpec][]);  
  println("Specification: ",iSpec_label);
  decl seed=floor(timeofday()*257);
  decl vSeed=ranseed(0);
  ranseed(vSeed[seed]);
  decl initial_rv=ranu(iSolutionSeed,1);

  /**********************************************************/
  /* Parameters: Specification */
  iTaxDelay=2;
  iVariety=1;
  iConvenience=0;
  iEstimator="NLS";
  iClusterThreshold=20;
  decl demand_spec=2;
  decl OPTid=<1;2;3>;
  decl iCluster=0;
  //vParam_logistic=loadmat(sprint("Results/matching_function_param_",iEstimator,"_variety",iVariety,"_spec",demand_spec,".mat"));
  vParam_logistic=loadmat(sprint("Results/matching_function_stockout_param_",iEstimator,"_variety",iVariety,"_spec",demand_spec,".mat"));
  decl mParam_cost_CI=loadmat(sprint("Results/CI_cost_nbparam_",rows(OPTid),"_taxdelay",iTaxDelay,"_",iEstimator,"_cluster",iCluster,".csv"));
  println("Param CI: ",mParam_cost_CI);
  vParam_cost=new matrix[4][1];
  vParam_cost=mParam_cost_CI[][0];
  println("param_cost: ",vParam_cost);
  println("shipping cost param (dist): ",vParam_cost[iShip_dist]);
  println("shipping cost param (vi): ",vParam_cost[iShip_vi]);
  println("shipping cost param (density): ",vParam_cost[iFC_dens]);
  
  /* Tax Status */
  iTaxDelay=iSolutionTax;
    
  println("%r",aParamName,vParam_logistic);
  println("Tax Delay indicator : ",iTaxDelay);
  println("Variety: ",iVariety);
  println("Convenience: ",iConvenience);
  println("iEstimator: ",iEstimator);
  println("iSortThreshold: ",iSortThreshold);
  println("iFCThreshold: ",iFCThreshold);
  println("iClusterThreshold: ",iClusterThreshold);
  println("iDummy_SC: ",iDummy_SC);
  
  /**********************************************************/
  /* Load Data-sets: Fixed covariates */
  decl data_dir="data_files/";
  scale=1000000; /* Capacity and revenue scale variable */
  decl aVarName_cross,aVarName_cluster,aVarName_distance;
  decl mCrossData=loadmat(sprint(data_dir,"Distance_matrix_county_cluster_combined_threshold",iClusterThreshold,".dta"),&aVarName_cross);
  decl mDistanceData=loadmat(sprint(data_dir,"Cluster_combined_distance_threshold",iClusterThreshold,".dta"),&aVarName_distance);
  decl aVarName_demo;
  decl mDemoData=loadmat(sprint(data_dir,"County_demo_variables.dta"),&aVarName_demo);
  mDemoData=sortbyc(mDemoData,<demo_county_code,demo_year>);  
  println("%r",aVarName_demo,(meanc(mDemoData)|sqrt(varc(mDemoData)))');  
  decl mClusterData=loadmat(sprint(data_dir,"Cluster_panel_threshold",iClusterThreshold,".dta"),&aVarName_cluster);
  
  decl aVarName_variety;
  decl mVariety_index=loadmat(sprint(data_dir,"Variety_index_spec",demand_spec,".dta"),&aVarName_variety);
  println("%c",aVarName_variety,mVariety_index);
  vVariety=mVariety_index[][1]';

  decl mSpendingComponents=loadmat(sprint(data_dir,"ces_data",demand_spec,".csv"));
  mSpendingComponents=sortbyc(mSpendingComponents,<demand_mode,demand_county,demand_year>);
  println("unique modes: ",unique(mSpendingComponents[][demand_mode]));
  decl mSpendingComponents_mode1=selectifr(mSpendingComponents,mSpendingComponents[][demand_mode].==1);
  decl mSpendingComponents_mode2=selectifr(mSpendingComponents,mSpendingComponents[][demand_mode].==2);
  decl mSpendingComponents_mode3=selectifr(mSpendingComponents,mSpendingComponents[][demand_mode].==3);
  
  /* Sort cross-data by county and cluster ID */
  mCrossData=sortbyc(mCrossData,<cross_county_code,cross_cluster_id>);
  mDistanceData=sortbyc(mDistanceData,<cluster_id0,cluster_id1>);
  mClusterData=sortbyc(mClusterData,<cluster_id,cluster_year>);
  
  /* Vector of IDs */
  vCluster_id=unique(mCrossData[][cross_cluster_id]);
  vCounty_id=unique(mCrossData[][cross_county_code]);
  vYear=unique(mDemoData[][demo_year]);
  N=columns(vCluster_id);
  M=columns(vCounty_id);
  T=columns(vYear);
  println("N : ",N);
  println("M : ",M);
  println("T : ",T);

  /* Reshape wide distance matrix */
  mDistance_cluster=reshape(mDistanceData[][cluster_dist],N,N);
  mDist_county_cluster=reshape(mCrossData[][cross_dist],M,N)+iMinDist;
  mDummy_in_network=(mDist_county_cluster.<iSortThreshold);
  mInvDist_county_cluster=exp(-mDist_county_cluster);
  println("mDistance_cluster: ",mDistance_cluster[:10][:10]);
  
  /* Reshape input price and tax variables */
  mCounty_wage=reshape(mDemoData[][demo_wage],M,T); mCounty_wage=meanr(mCounty_wage).*ones(M,T);
  mCounty_rent=reshape(mDemoData[][demo_rent],M,T); mCounty_rent=meanr(mCounty_rent).*ones(M,T);
  mCounty_density=reshape(mDemoData[][demo_pop_density],M,T); mCounty_density=meanr(mCounty_density).*ones(M,T);
  mCounty_hh=reshape(mDemoData[][demo_nb_hh],M,T);
  mCounty_tax=reshape(mDemoData[][demo_tax],M,T);
  mCounty_high_income=reshape(mDemoData[][demo_hhincome_100000more],M,T).*mCounty_hh;
  mCounty_age=reshape(mDemoData[][demo_mean_age],M,T);
  mCounty_share_young=reshape(mDemoData[][demo_share_young],M,T);
  mCounty_pop_young=reshape(mDemoData[][demo_pop_young],M,T);
  decl iAvgDensity=sumc(mCounty_density[][T-1].*mCounty_hh[][T-1])./sumc(mCounty_hh[][T-1]);
  vState_code=reshape(mDemoData[][demo_state_code],M,T)[][0];
  println("%c",{"year","variety ratio"},(vYear|vVariety)');

  /* Reshape wide */
  iMu_param=unique(mSpendingComponents_mode1[][demand_margin0]);
  iMu_amazon=unique(mSpendingComponents_mode1[][demand_a_margin]);
  iMu_mp=unique(mSpendingComponents_mode1[][demand_mp_margin]);
  println("Profit margin: ",iMu_param);
  decl mSpendingAlpha_mode1=reshape(mSpendingComponents_mode1[][demand_alpha],M,T);
  decl mSpendingAlpha_mode2=reshape(mSpendingComponents_mode2[][demand_alpha],M,T);
  decl mSpendingAlpha_mode3=reshape(mSpendingComponents_mode3[][demand_alpha],M,T);
  aSpendingInt=new array[3];
  aSpendingInt[0]=mSpendingAlpha_mode1;
  aSpendingInt[1]=mSpendingAlpha_mode2;
  aSpendingInt[2]=mSpendingAlpha_mode3;
  
  /* Construct matrices of amazon characteristics */
  //vAmazonShare=reshape(mSpendingComponents_mode1[][demand_am_share],M,T)[0][];
  vAmazonShare=ones(1,T); /* This restricts all amazon transactions to be taxable */
  mRetailSpending=reshape(mSpendingComponents_mode1[][demand_avg_spending_annual_county],M,T);
  mAmazonPrice_tax_excl=reshape(mSpendingComponents_mode1[][demand_pre_laprice1],M,T);
  decl vAvgPrice_tax_excl=sumc(mAmazonPrice_tax_excl.*mCounty_hh)./sumc(mCounty_hh);
  println("Population weighted average price: ");
  println("%c",{"year","avg. price","variety"},vYear'~vAvgPrice_tax_excl'~vVariety');  
  decl mCounty_per_100=reshape(mSpendingComponents_mode1[][demand_perc_100],M,T);
  decl iCES_coef_int=unique(mSpendingComponents_mode1[][demand_tax_coeff]);
  decl iCES_coef_slope=unique(mSpendingComponents_mode1[][demand_tax_coeff100]);
  vNetwork_density_obs=reshape(mSpendingComponents_mode1[][demand_density1],M,T)[0][];
  iDensityParam=unique(mSpendingComponents_mode1[][demand_density_coeff]);
  println("iCES_coef_int: ",iCES_coef_int);
  println("iCES_coef_slope: ",iCES_coef_slope);
  decl max_inc_share=(1.1-iCES_coef_int)/iCES_coef_slope;
  println("max_inc_share: ",max_inc_share);
  mCES_param=1-iCES_coef_int-iCES_coef_slope.*setbounds(mCounty_per_100,0,max_inc_share);  
  decl mAmazon_tax_obs=reshape(mDemoData[][demo_amazon_tax],M,T);
  println("Density parameter: ",iDensityParam);
  println("network density:: ",(vYear|vNetwork_density_obs)');

  /***************************************************************************/
  /* Calculate input prices for each locations */
  mCluster_wage=reshape(mClusterData[][cluster_imputed_wage],N,T)[][T-1].*ones(N,T);
  mCluster_rent=reshape(mClusterData[][cluster_county_rent_dis_imputed],N,T)[][T-1].*ones(N,T);
  mCluster_density=reshape(mClusterData[][cluster_pop_density],N,T).*ones(N,T);
  decl vCluster_county_code=reshape(mClusterData[][cluster_county_code],N,T)[][0];
  mDummy_cluster_county=vCluster_county_code.==vCounty_id;
  println("cluster cost distribution: ","%c",{"wage","rent","density"},quantilec(mCluster_wage[][0]~mCluster_rent[][0]~mCluster_density[][0],<0,1/4,1/2,3/4,1>));

  /************************************************************************/
  /* Evalute profit for candidate networks */
  year=vecindex(vYear,iSolutionYear);
  decl vFChat,vSChat;
  decl u1=2*ranu(1,1)-1;
  decl u2=2*ranu(1,1)-1;  
  if(vYear[year].==1999) {
    vFChat=constant((3+3*u1),N,1);
    vSChat=constant((1+1*u2),N,1);
    println("random n: ",3+3*u1);
  }
  if(vYear[year].==2006) {
    vFChat=constant((10+10*u1),N,1);
    vSChat=constant((3+3*u2),N,1);
    println("random n: ",10+10*u1);
  }
  if(vYear[year].==2012) {
    vFChat=constant((30+30*u1),N,1);
    vSChat=constant((5+5*u2),N,1);
    println("random n: ",30+30*u1);
  }
  if(vYear[year].==2018) {
    vFChat=constant((80+80*u1),N,1);
    vSChat=constant((30+30*u2),N,1);
    println("random n: ",80+80*u1);
  }
  vFChat./=N;
  vSChat./=N;
  println("expected number of facilities (starting values): ",sumc(vFChat)~sumc(vSChat));
  
  decl vParam0=vFChat|vSChat;
  iprint=0;
  decl PI;
  decl time0=timer();
  decl iTargetVolume;
  decl iMaxProfit,iAvgProfit,vAvgCost,vLogistic,vTax,iQ;
  decl vGrid_AC=<>;
  decl mResults=<>;
  decl maxid;
  decl vNetwork;
  decl vNetwork_fc;
  decl mCluster_location,mCluster_characteristics;
  println("year: ",vYear[year]);
  vGrid_AC=<>;
  mResults=<>;
  
  /* Tuning parameters */
  decl vTuning_param_pbil=new matrix[nbTuning_param];
  vTuning_param_pbil[iLR_fc]=0.1;
  vTuning_param_pbil[iLR_sc]=0.1;
  vTuning_param_pbil[iMUT_PROB]=0.02;
  vTuning_param_pbil[iMUT_SHIFT]=0.05;
  vTuning_param_pbil[iMAXIT]=500;
  vTuning_param_pbil[iSequential_update]=1;
  vTuning_param_pbil[iSim]=1000;
  vTuning_param_pbil[inbRuns]=1;
  vTuning_param_pbil[iGreedy]=1;
  decl vGrid_target_volume=exp(range(log(55),log(10000),0.1));

  /****************************************************/
  /* Test the profit and cost functions */
  iTargetVolume=vGrid_target_volume[floor(columns(vGrid_target_volume)/2)];
  println("target: ",iTargetVolume);
  mCluster_location=zeros(N,2);
  for(i=0;i<N;i++) mCluster_location[i][0]=ranpoissonprocess(1,1,vFChat[i]);
  println("nb of FCs:" ,sumc(mCluster_location[][0]));
  mCluster_characteristics=mCluster_location~(mCluster_location[][0].*iFC_cap)~(mCluster_location[][1].*iSC_cap);  
  cost_components(mCluster_characteristics,&iQ,&iAvgProfit,&vAvgCost,&vLogistic,&vTax,iTargetVolume);
  println("avg cost: ",vAvgCost);
  println("iAvg profit: ",iAvgProfit);
  println("logistic: ",vLogistic);
  println("tax: ",vTax);  
  profit(mCluster_characteristics,&PI);
  println("PI : ",PI);

  /****************************************************/  
  /* Solve optimal network */
  pbil_max(&vNetwork,&iMaxProfit,vParam0,vTuning_param_pbil);
  vNetwork_fc=vNetwork[:N-1];
  mCluster_location=vNetwork[:N-1]~vNetwork[N:];
  mCluster_characteristics=mCluster_location~(mCluster_location[][0].*iFC_cap)~(mCluster_location[][1].*iSC_cap);    
  savemat(sprint("Network_solution_spec",iSpec_label,".dta"),vCluster_id'~mCluster_location~constant(iMaxProfit,N,1),{"cluster_id","nb_fc","nb_sc","total_profit"});
}

