#delimit;
clear;
set more off;
/* 
This file extracts the optimal network configuration from the simulation files.
*/
#delimit;
global computer "Local";
*global computer "Linstat";
if "$computer" == "Local" {;
	local intermediate_dir "~/Dropbox/Amazon_FC/Data/Processed/Intermediate";
	local swap_dir "~/Dropbox/Amazon_FC/Data/Processed/Swap_list";
	local main_dir "~/Dropbox/Amazon_FC/Data/Processed/Main";
	local county_dir "~/Dropbox/Amazon_FC/Data/County_Characteristics";
	local FC_dir "~/Dropbox/Amazon_FC/Data/FCs";
	local tables_dir "~/Dropbox/Amazon_FC/Tables_and_Figures";
	local htc "~/Dropbox/Amazon_FC/Analysis/Oxfiles/OxHTC";
	};
if "$computer" == "Linstat" {;
	local intermediate_dir "/project/houde/Amazon_FC/Data/Processed/Intermediate";
	local swap_dir "/project/houde/Amazon_FC/Data/Processed/Swap_list";
	local main_dir "/project/houde/Amazon_FC/Data/Processed/Main";
	local county_dir "/project/houde/Amazon_FC/Data/County_Characteristics";
	local FC_dir "/project/houde/Amazon_FC/Data/FCs";
	local htc "project/houde/Amazon_FC/Analysis/Oxfiles/OxHTC";
	};
set more off;

cd "`htc'/archive_dta/nov10";
local threshold=20;
local sig=1/100;


/* Location of maximum profit network */
foreach year in 1999 2006 2012 2018 {;
foreach tax in 0 1  {;
	use Network_simulation_year`year'_taxdelay`tax', clear;
	format * %9.6g;
	drop if total_profit==0 | error==1;
	bysort  spec: egen double sd_total_profit=sd(total_profit);
	bysort  spec: egen double total_sc=sum(nb_sc);
	tab total_sc;
	drop total_sc;
	egen double max_profit=max(total_profit);
	gen cv_total_profit=sd_total_profit/max_profit;
	drop sd_total_profit;
	gen dummy_max=abs(total_profit-max_profit)<10^(-6);
	tab dummy_max;
	keep if dummy_max==1;	
	bysort cluster_id: egen rank_spec=rank(spec), field;
	keep if rank_spec==1;
	drop rank_spec max_profit dummy_max;
	egen total_fc=sum(nb_fc);
	egen total_sc=sum(nb_sc);
	merge 1:m cluster_id year using "`main_dir'/Cluster_panel_threshold`threshold'";
	keep if _merge==3;
	drop _merge;
	merge m:1 state_code using "`intermediate_dir'/State_census_codes";
	keep if _merge==3;
	drop _merge;
	save "`main_dir'/Network_maximum_year`year'_taxdelay`tax'", replace;
};

use "`main_dir'/Network_maximum_year`year'_taxdelay0", clear;
rename nb_fc nb_fc0;
rename nb_sc nb_sc0;
rename total_fc total_fc0;
rename total_sc total_sc0;
merge 1:1  cluster_id  using "`main_dir'/Network_maximum_year`year'_taxdelay1";
rename nb_fc nb_fc1;
rename nb_sc nb_sc1;
rename total_fc total_fc1;
rename total_sc total_sc1;
gen change_nb_fc=nb_fc1-nb_fc0;
gen change_nb_sc=nb_sc1-nb_sc0;
gen change_total_fc=total_fc1-total_fc0;
gen change_total_sc=total_sc1-total_sc0;
drop _merge;
save "`main_dir'/Network_maximum_year`year'", replace;
};




/* Average number of facilities per cluster */
foreach year in 1999 2006 2012 2018 {;
foreach tax in 0 1  {;
	use Network_simulation_year`year'_taxdelay`tax', clear;
	format * %9.6g;
	drop if total_profit==0 | error==1;
	bysort cluster_id: egen double mean_profit=mean(total_profit);
	replace total_profit=total_profit/mean_profit;
	tabstat total_profit, stat(mean sd min p25 p50 p75 max);	
	replace total_profit=exp(total_profit/`sig');	
	bysort cluster_id: egen double prob=sum(total_profit);
	replace prob=total_profit/prob;
	tabstat prob, stat(mean sd min p25 p50 p75 max);	
	replace nb_fc=nb_fc*prob;
	replace nb_sc=nb_sc*prob;
	collapse (sum) nb_fc nb_sc, by(year taxdelay cluster_id);		
	egen total_fc=sum(nb_fc);
	egen total_sc=sum(nb_sc);
	merge 1:m cluster_id year using "`main_dir'/Cluster_panel_threshold`threshold'";
	keep if _merge==3;
	drop _merge;
	merge m:1 state_code using "`intermediate_dir'/State_census_codes";
	keep if _merge==3;
	drop _merge;
	save "`main_dir'/Network_average_year`year'_taxdelay`tax'", replace;
};

use "`main_dir'/Network_average_year`year'_taxdelay0", clear;
rename nb_fc nb_fc0;
rename nb_sc nb_sc0;
rename total_fc total_fc0;
rename total_sc total_sc0;
merge 1:1  cluster_id  using "`main_dir'/Network_average_year`year'_taxdelay1";
rename nb_fc nb_fc1;
rename nb_sc nb_sc1;
rename total_fc total_fc1;
rename total_sc total_sc1;
gen change_nb_fc=nb_fc1-nb_fc0;
gen change_nb_sc=nb_sc1-nb_sc0;
gen change_total_fc=total_fc1-total_fc0;
gen change_total_sc=total_sc1-total_sc0;
drop _merge;
save "`main_dir'/Network_average_year`year'", replace;
};


/* Average number of facilities per state */
foreach year in 1999 2006 2012 2018  {;
foreach tax in 0 1   {;
	use Network_simulation_year`year'_taxdelay`tax', clear;
	format * %9.6g;
	drop if total_profit==0 | error==1;
	bysort cluster_id: egen double mean_profit=mean(total_profit);
	replace total_profit=total_profit/mean_profit;
	tabstat total_profit, stat(mean sd min p25 p50 p75 max);	
	replace total_profit=exp(total_profit/`sig');	
	bysort cluster_id: egen double prob=sum(total_profit);
	replace prob=total_profit/prob;
	tabstat prob, stat(mean sd min p25 p50 p75 max);	
	replace nb_fc=nb_fc*prob;
	replace nb_sc=nb_sc*prob;
	collapse (sum) nb_fc nb_sc, by(year taxdelay cluster_id);	
	merge 1:m cluster_id year using "`main_dir'/Cluster_panel_threshold`threshold'";
	keep if _merge==3;
	drop _merge;
	merge m:1 state_code using "`intermediate_dir'/State_census_codes";
	keep if _merge==3;
	drop _merge;
	collapse (sum) nb_fc nb_sc (mean) cluster_lat cluster_long , by(state_code state_abbrev);
	save "`main_dir'/Solution_state_presence_year`year'_taxdelay`tax'", replace;
};

use "`main_dir'/Solution_state_presence_year`year'_taxdelay0", clear;
rename nb_fc nb_fc0;
rename nb_sc nb_sc0;
merge 1:1  state_code  using "`main_dir'/Solution_state_presence_year`year'_taxdelay1";
rename nb_fc nb_fc1;
rename nb_sc nb_sc1;
gen change_nb_fc=nb_fc1-nb_fc0;
gen change_nb_sc=nb_sc1-nb_sc0;
drop _merge;
save "`main_dir'/Solution_state_presence_year`year'", replace;
};
