/* Matching probability function */
match_prob(aFunc,const vActive_id,const vMu)
{
  //decl M=rows(mDist_county_cluster);
  //decl N=columns(mDist_county_cluster);
  decl n=rows(vActive_id);
  decl mMu=ones(M,N).*vMu';
  decl mMatchProb=new matrix[M][N]; /* Match probability matrix (columns = cluster id) */
  decl mDummy_lower,mDummy_higher;
  decl id_lower,id;
  decl j;
  if(vParam_logistic[imu_0]==100) {
    decl vNearestCluster=mincindex(mDist_county_cluster[][vActive_id]');
    parallel for(j=0;j<M;j++) mMatchProb[j][vActive_id[vNearestCluster[j]]]=1;
  }
  else {
    parallel for(j=0;j<n;j++)
      {
	mDummy_lower=mDist_county_cluster[][vActive_id].<mDist_county_cluster[][vActive_id[j]];
	mDummy_higher=1-mDummy_lower;
	mMatchProb[][vActive_id[j]]=vMu[vActive_id[j]].*prodr((1-vMu[vActive_id]').*mDummy_lower+mDummy_higher);
      }
    mMatchProb./=sumr(mMatchProb);
  }
  aFunc[0]=mMatchProb;
  return 1;
}
sortation_network(aNetwork_sortation,aSortation_zone,aDist_fc_sc,const mCluster_characteristics)
{
  decl mCluster_location=mCluster_characteristics[][<0,1>];
  decl mCluster_capacity=mCluster_characteristics[][<2,3>];
  decl vActive_cluster_fc=vecindex(mCluster_location[][0].>0,1);
  decl vActive_cluster_sc=vecindex(mCluster_location[][1].>0,1);
  /* Distance between FC and Sortation facilities */
  decl mDist_fc_sort=mDistance_cluster[vActive_cluster_fc][vActive_cluster_sc];
  /* Set = FCs within iFCThreshold miles: Nb FC x Nb SC */
  decl mNetwork_sortation;
  if(iNearestFC) mNetwork_sortation=setbounds((mDist_fc_sort.<=iFCThreshold)+(mDist_fc_sort.==minc(mDist_fc_sort)),0,1);
  else mNetwork_sortation=setbounds(mDist_fc_sort.<=iFCThreshold,0,1);
  /* Sortation network */
  decl mSortation_zone=(mCluster_location[][1]').*mDummy_in_network;
  mSortation_zone.*=mInvDist_county_cluster;
  mSortation_zone./=sumr(mSortation_zone);
  mSortation_zone.*=mCluster_location[][1]'; 
  decl vNaNid=vecindex(sumr(mSortation_zone),.NaN);
  mSortation_zone[vNaNid][]=0;
  aNetwork_sortation[0]=mNetwork_sortation;
  aSortation_zone[0]=mSortation_zone;
  aDist_fc_sc[0]=mDist_fc_sort;
}
/********************************************************************************************************************************************************/
/* Logistic model: Match probability and output functions */
logistic_func(aOutput,aDist,aLabor,aMatchProb,const mCluster_characteristics,const vCounty_demand)
{
  decl mCluster_location=mCluster_characteristics[][<0,1>];
  decl mCluster_capacity=mCluster_characteristics[][<2,3>];  
  decl vActive_cluster_fc=vecindex(mCluster_location[][0].>0,1);
  decl vActive_cluster_sc=vecindex(mCluster_location[][1].>0,1);
  /***************************************************/
  /* Sortation network */
  decl mNetwork_sortation,mSortation_zone,mDist_fc_sc;
  sortation_network(&mNetwork_sortation,&mSortation_zone,&mDist_fc_sc,mCluster_characteristics);
  /* Indicator variable equal to one with a county and FCs are connected: Nb Counties x Nb of Active Locations */
  decl mSortationNetwork=setbounds(mSortation_zone[][vActive_cluster_sc]*mNetwork_sortation',0,1);  
  /* Capacity of clusters: Nx1 */
  decl theta;
  if(iVariety) theta=vParam_logistic[imu_0]/vVariety[year];
  else theta=vParam_logistic[imu_0];
  decl vMu=zeros(N,1);
  decl vCluster_capacity=mCluster_capacity[][0];
  vMu[vActive_cluster_fc]=1-exp(-theta.*vCluster_capacity[vActive_cluster_fc]);
  decl n=rows(vActive_cluster_fc);

  /***************************************************/  
  /* Matching probability */
  decl mMatchProb;
  match_prob(&mMatchProb,vActive_cluster_fc,vMu);
  
  /***************************************************/  
  /* Output: FC */
  decl mCluster_demand=vCounty_demand.*mMatchProb;
  decl vOutput_fc=sumc(mCluster_demand[][vActive_cluster_fc])';
  //decl vOutput_fc=mMatchProb[][vActive_cluster_fc]'vCounty_demand;
  /* Output: SC */
  /* Sum across counties of volume potentially transiting through each sortation: Nb. Active FC x Nb SC */
  decl mOrder_fc_sc=(mCluster_demand)'mSortation_zone;
  decl vOutput_sc=sumc(mNetwork_sortation.*mOrder_fc_sc[vActive_cluster_fc][vActive_cluster_sc])';
  decl mOutput=zeros(N,2);
  mOutput[vActive_cluster_fc][0]=vOutput_fc;
  mOutput[vActive_cluster_sc][1]=vOutput_sc;

  /***************************************************/
  /* Labor demand: Per facility */
  decl mLabor=zeros(N,2);
  mLabor[vActive_cluster_fc][0]=100*(vParam_logistic[iAfc]).*((vOutput_fc./mCluster_location[vActive_cluster_fc][0]).^vParam_logistic[ialpha]);
  mLabor[vActive_cluster_sc][1]=100*(vParam_logistic[iAsc]).*(vOutput_sc.^vParam_logistic[ialpha]);
  /***************************************************/
  /* Distance and VI output */
  decl vOutput_vi;
  if(vActive_cluster_sc!=<>) vOutput_vi=sumc(mCluster_demand[][vActive_cluster_fc].*mSortationNetwork)';
  else vOutput_vi=0;
  decl vShipping_dist=sumc(mCluster_demand[][vActive_cluster_fc].*mDist_county_cluster[][vActive_cluster_fc])';
  /***************************************************/
  /* Store results */
  aOutput[0]=mOutput;
  aLabor[0]=mLabor;
  aMatchProb[0]=mMatchProb;
  decl mDist=new matrix[N][2];
  if(vActive_cluster_fc==<>) {
    mDist=constant(0,N,1)~constant(M_INF_POS,N,1);
  }
  else {
    if(vActive_cluster_sc==<>) {
      mDist[vActive_cluster_fc][]=constant(0,vActive_cluster_fc)~vShipping_dist;
    }
    if(vActive_cluster_sc!=<>) {
      mDist[vActive_cluster_fc][]=(vOutput_vi~vShipping_dist);
    }    
  }
  aDist[0]=mDist;
  return 1;
}
profit(const mCluster_characteristics,const aPI)
{
  decl mCluster_location=mCluster_characteristics[][<0,1>];
  decl mCluster_capacity=mCluster_characteristics[][<2,3>];  
  /* Revenue: Tax and Orders */
  decl vCounty_presence=(sumr(mCluster_location)'mDummy_cluster_county)';
  vCounty_presence=vCounty_presence.>0;
  decl vAmazonSpending,vCounty_demand,vAmazonNetRevenue,vAmazon_tax;  
  amazon_tax_func(&vAmazon_tax,vCounty_presence);
  spending_func(&vAmazonSpending,&vAmazonNetRevenue,&vCounty_demand,vAmazon_tax,vNetwork_density_obs);
  vCounty_demand.*=mCounty_hh[][year]/scale;
  vAmazonNetRevenue.*=mCounty_hh[][year]/scale;
  /* Output: Distance, output and labor demand functions per facility/location */
  decl mOutput,mDist,mLabor,mMatchProb;  
  logistic_func(&mOutput,&mDist,&mLabor,&mMatchProb,mCluster_characteristics,vCounty_demand);
  if(iprint) {
    println("%r",{"net rev","orders","hh","avg order","avg dist"},(sumc(vAmazonNetRevenue)~sumc(vCounty_demand)~sumc(mCounty_hh[][year]/scale)
								   ~meanc(vCounty_demand./(mCounty_hh[][year]/scale))~(sumc(mDist[][1])/sumc(vCounty_demand)))');
  }
  /* Variable and fixed costs */
  decl vLabor_cost=(mCluster_location[][0].*mLabor[][0].*mCluster_wage[][year]
		    +mLabor[][1].*mCluster_wage[][year])/scale;
  decl vZ=sumc(mOutput[][0])~(sumc(mDist[][1])/100)~sumc(mDist[][0]);
  decl iShip_cost=vZ*vParam_cost[<iShip_int,iShip_dist,iShip_vi>];
  decl mDensityCap=(mCluster_capacity[][0].*mCluster_density[][year])~(mCluster_capacity[][1].*mCluster_density[][year]);
  decl mLandCap=(mCluster_capacity[][0].*mCluster_rent[][year])~(mCluster_capacity[][1].*mCluster_rent[][year]);
  decl vLand_cost=sumr(mLandCap);
  decl vDensity_cost=sumr(mDensityCap)/100;
  decl vCluster_cost=sumr(vLabor_cost+vLand_cost+vDensity_cost*vParam_cost[iFC_dens]);
  /* Total profits */
  decl PI=sumc(vAmazonNetRevenue)-iShip_cost-sumc(vCluster_cost);
  if(iprint) {
    println("%r",{"Profit","Net revenue","Sipping cost","Wage","Rent","Density","VI Share","SC/order","Nb of FCs","Nb of SCs"},"%cf",{"%13.5g"},
	    (PI~sumc(vAmazonNetRevenue)~iShip_cost~sumc(vLabor_cost)~sumc(vLand_cost)~sumc(vDensity_cost*vParam_cost[iFC_dens])
	     ~sumc(mDist[][0])./sumc(vCounty_demand)~iShip_cost/sumc(vCounty_demand)~sumc(mCluster_location[][0])~sumc(mCluster_location[][1]))');
  }
  aPI[0]=PI;
  return 1;
}
facility_profit(const mCluster_characteristics,const aCounty_demand,const aPI)
{
  decl mCluster_location=mCluster_characteristics[][<0,1>];
  decl mCluster_capacity=mCluster_characteristics[][<2,3>];
  /* Revenue: Tax and Orders */
  decl vCounty_presence=(sumr(mCluster_location)'mDummy_cluster_county)';
  vCounty_presence=vCounty_presence.>0;
  decl vAmazonSpending,vCounty_demand,vAmazonNetRevenue,vAmazon_tax;  
  amazon_tax_func(&vAmazon_tax,vCounty_presence);
  spending_func(&vAmazonSpending,&vAmazonNetRevenue,&vCounty_demand,vAmazon_tax,vNetwork_density_obs);
  vCounty_demand.*=mCounty_hh[][year]/scale;
  vAmazonNetRevenue.*=mCounty_hh[][year]/scale;

  /* Output: Distance, output and labor demand functions per facility/location */
  decl mOutput,mDist,mLabor,mMatchProb;
  logistic_func(&mOutput,&mDist,&mLabor,&mMatchProb,mCluster_characteristics,vCounty_demand);

  /* Variable and fixed costs */
  decl vFacilityNetRevenue=mMatchProb'vAmazonNetRevenue;
  decl vFacilityOutput=mOutput[][0];
  decl vLabor_cost=(mCluster_location[][0].*mLabor[][0].*mCluster_wage[][year]
		    +mLabor[][1].*mCluster_wage[][year])/scale;
  decl mZ=(mOutput[][0])~(mDist[][1]/100)~(mDist[][0]);
  decl vShip_cost=mZ*vParam_cost[<iShip_int,iShip_dist,iShip_vi>];
  decl mDensityCap=(mCluster_capacity[][0].*mCluster_density[][year])~(mCluster_capacity[][1].*mCluster_density[][year]);
  decl mLandCap=(mCluster_capacity[][0].*mCluster_rent[][year])~(mCluster_capacity[][1].*mCluster_rent[][year]);
  decl vLand_cost=sumr(mLandCap);
  decl vDensity_cost=(sumr(mDensityCap)/100)*vParam_cost[iFC_dens];
  decl vFixed_cost=vLand_cost+vDensity_cost;
  /* Profit components */
  decl mPI=vFacilityOutput~vFacilityNetRevenue~vShip_cost~vLabor_cost~vLand_cost~vDensity_cost~(vShip_cost+vLabor_cost+vFixed_cost);
  aPI[0]=mPI;
  aCounty_demand[0]=vCounty_demand;
  return 1;
}
cost_components(const mCluster_characteristics,const aOutput,const aAvgProfit,const aAvgCost,const aLogistic,const aTax,const iTargetVolume)
{
  decl mCluster_location=mCluster_characteristics[][<0,1>];
  decl mCluster_capacity=mCluster_characteristics[][<2,3>];    
  /* Revenue: Tax and Orders */
  decl vCounty_presence=(sumr(mCluster_location)'mDummy_cluster_county)';
  vCounty_presence=vCounty_presence.>0;
  decl vAmazonSpending,vCounty_demand,vAmazonNetRevenue,vAmazon_tax;  
  amazon_tax_func(&vAmazon_tax,vCounty_presence);
  spending_func(&vAmazonSpending,&vAmazonNetRevenue,&vCounty_demand,vAmazon_tax,vNetwork_density_obs);
  vCounty_demand=(vCounty_demand.*mCounty_hh[][year]/scale);
  vAmazonNetRevenue=(vAmazonNetRevenue.*mCounty_hh[][year]/scale);
  decl iMultiplier;
  if(iTargetVolume==<>) iMultiplier=1;
  else iMultiplier=iTargetVolume/sumc(vCounty_demand);
  vCounty_demand.*=iMultiplier;
  vAmazonNetRevenue.*=iMultiplier;
  println("log total volume: ",log(sumc(vCounty_demand)));
  
  decl iAvgAmazonTax=sumc(mCounty_hh[][year].*vAmazon_tax)./sumc(mCounty_hh[][year]);
  decl iFractionTaxable=sumc(mCounty_hh[][year].*(vAmazon_tax.>0))./sumc(mCounty_hh[][year]);
  
  /* Output: Distance, output and labor demand functions per facility/location */
  decl mOutput,mDist,mLabor,mMatchProb;  
  logistic_func(&mOutput,&mDist,&mLabor,&mMatchProb,mCluster_characteristics,vCounty_demand);
  if(iprint) {
    println("%r",{"net rev","orders","hh","avg order","avg dist"},(sumc(vAmazonNetRevenue)~sumc(vCounty_demand)~sumc(mCounty_hh[][year]/scale)
								   ~meanc(vCounty_demand./(mCounty_hh[][year]/scale))~(sumc(mDist[][1])/sumc(vCounty_demand)))');
  }
  /* Variable and fixed costs */
  decl vLabor_cost=(mCluster_location[][0].*mLabor[][0].*mCluster_wage[][year]
		    +mLabor[][1].*mCluster_wage[][year])/scale;
  decl vZ=sumc(mOutput[][0])~(sumc(mDist[][1])/100)~sumc(mDist[][0]);
  decl iShip_cost=vZ*vParam_cost[<iShip_int,iShip_dist,iShip_vi>];
  decl mDensityCap=(mCluster_capacity[][0].*mCluster_density[][year])~(mCluster_capacity[][1].*mCluster_density[][year]);
  decl mLandCap=(mCluster_capacity[][0].*mCluster_rent[][year])~(mCluster_capacity[][1].*mCluster_rent[][year]);
  decl vLand_cost=sumr(mLandCap);
  decl vDensity_cost=sumr(mDensityCap)/100;
  decl vCluster_cost=sumr(vLabor_cost+vLand_cost+vDensity_cost*vParam_cost[iFC_dens]);
  /* Total profits */
  decl PI=sumc(vAmazonNetRevenue)-iShip_cost-sumc(vCluster_cost);  
  if(iprint) {
    println("%r",{"Profit","Net revenue","Sipping cost","Wage","Rent","Density","VI Share","SC/order","Nb of FCs","Nb of SCs"},"%cf",{"%13.5g"},
	    (PI~sumc(vAmazonNetRevenue)~iShip_cost~sumc(vLabor_cost)~sumc(vLand_cost)~sumc(vDensity_cost*vParam_cost[iFC_dens])
	     ~sumc(mDist[][0])./sumc(vCounty_demand)~iShip_cost/sumc(vCounty_demand)~sumc(mCluster_location[][0])~sumc(mCluster_location[][1]))');
  }
  /* Store results */
  decl vCost_components=new matrix[4][1];  
  decl Q=sumc(vCounty_demand);
  vCost_components[0]=iShip_cost/Q;
  vCost_components[1]=sumc(vLabor_cost)/Q;
  vCost_components[2]=sumc(vDensity_cost*vParam_cost[iFC_dens])/Q;
  vCost_components[3]=sumc(vLand_cost)/Q;
  decl vLogistic_components=new matrix[2][1];
  vLogistic_components[0]=sumc(mDist[][1])/Q;
  vLogistic_components[1]=sumc(mDist[][0])/Q;
  aOutput[0]=Q;
  aAvgCost[0]=vCost_components;
  aLogistic[0]=vLogistic_components;
  aTax[0]=iAvgAmazonTax|iFractionTaxable;
  aAvgProfit[0]=PI/Q;
  return 1;
}
sim_profit(const aSimPI,const aParam_range,const vParam,const iSequential_update,const Sim)
{
  decl vFChat=vParam[:N-1];
  decl vSChat=vParam[N:];
  decl mCluster_location=new matrix[N][2];
  decl mCluster_capacity=new matrix[N][2];
  decl mCluster_characteristics=new matrix[N][4];
  decl aSimLocation=new array[Sim];
  decl s,i,PI;
  decl vSimPI=new matrix[Sim][1];
  decl time0=timer();
  for(s=0;s<Sim;s++)
    {
      for(i=0;i<N;i++)
	{
	  if(iDummy_SC) {
	    mCluster_location[i][0]=ranpoissonprocess(1,1,vFChat[i]); 
	    if(iSequential_update) mCluster_location[i][1]=(ranu(1,1).<vSChat[i]).*(mCluster_location[i][0]>0);
	    else mCluster_location[i][1]=ranu(1,1).<vSChat[i];
	  }
	  else mCluster_location[i][]=(ranu(1,1).<vFChat[i])~0;
	}
      mCluster_capacity=(mCluster_location[][0]*iFC_cap)~(mCluster_location[][1]*iSC_cap);
      mCluster_characteristics=mCluster_location~mCluster_capacity;
      aSimLocation[s]=mCluster_characteristics;
    }
  decl time00;
  for(s=0;s<Sim;s++)
    {
      profit(aSimLocation[s],&PI);
      vSimPI[s]=PI;
    }
  //println("generation CPU sec: ",(timer()-time0)/100);
  decl iMaxid=maxcindex(vSimPI);
  decl iMinid=mincindex(vSimPI);
  decl vParam_max=(aSimLocation[iMaxid])[][0]|(aSimLocation[iMaxid])[][1];
  decl vParam_min=(aSimLocation[iMinid])[][0]|(aSimLocation[iMinid])[][1];
  aSimPI[0]=vSimPI;
  aParam_range[0]=vParam_max~vParam_min;
  return 1;
}
greedy_search(const avParam,const aPImax)
{
  decl mCluster_location0=(avParam[0])[:N-1]~(avParam[0])[N:];
  decl mCluster_location=mCluster_location0;
  decl mCluster_capacity_base=constant(iFC_cap,N,1)~constant(iSC_cap,N,1);
  decl mCluster_characteristics=new matrix[N][4];
  decl i,j;
  decl dPI,PI00,PI0,PI1;
  decl a,b=1;
  decl mA=<-1,-1,0,1,1>|<0,1,1,0,1>;
  decl K,id;
  decl it=0;
  decl iDisconnect;
  mCluster_characteristics=mCluster_location0~(mCluster_location0.*mCluster_capacity_base);
  profit(mCluster_characteristics,&PI0);
  decl time0=timer();
  decl maxit=1; /* Number of greedy steps */
  do{
    PI00=PI0;
    for(id=0;id<N;id++)
      {
	mCluster_location=mCluster_location0;
	for(a=0;a<columns(mA);a++)	 
	  {
	    /* Update FC */
	    mCluster_location[id][0]=mCluster_location0[id][0]+mA[0][a];
	    /* Update SC */
	    if(mA[1][a]) mCluster_location[id][1]=(1-mCluster_location0[id][1]);
	    else mCluster_location[id][1]=mCluster_location0[id][1];
	    /* Calculate profit if locations are non-negative and SCs are connected */
	    if(minr(mCluster_location[id][])>=0) {
	      iDisconnect=(mCluster_location[id][1].==1).*(mCluster_location[id][0].==0);
	      if(iDisconnect==0) {
		mCluster_characteristics=mCluster_location~(mCluster_location.*mCluster_capacity_base);	      
		profit(mCluster_characteristics,&PI1);
		if(PI1>PI0) {
		  mCluster_location0[id][]=mCluster_location[id][];
		  PI0=PI1;
		}
	      }
	    }
	  }
      }
    /* Update profit and network */
    it+=1;
    dPI=PI0-PI00;
  }while(it<maxit && dPI>0);
  avParam[0]=mCluster_location0[][0]|mCluster_location0[][1];
  aPImax[0]=PI0;
}


pbil_max(const aSolution_network,const aSolution_profits,const vParam0,const vT)
{
  decl LR_fc=vT[0];
  decl LR_sc=vT[1];
  decl MUT_PROB=vT[2];
  decl MUT_SHIFT=vT[3];
  decl MAXIT=vT[4];
  decl iSequential_update=vT[5];
  decl Sim=vT[6];
  decl nbRuns=vT[7];
  decl iGreedy=vT[8];
  println("Tuning parameters: ");
  print("%r",aTuning_param,vT);
  
  decl LR=constant(LR_fc,N,1)|constant(LR_sc,N,1);
  decl i,j,it,u;
  decl vSimPI,mParam_range;
  decl vParam;
  decl vNetwork;
  decl PImax;
  decl dT;
  decl dObj=-1000;
  decl vParam_start=vParam0;
  decl vParam_max,LocalMaxPI;
  decl time0;
  
  vParam=vParam_start;
  vNetwork=vParam_start;
  PImax=-10000;
  it=0;
  dT=0;
  /* Load current position */
  decl vPosition=loadmat(sprint("position_temp_spec",iSpec_label,".mat"));
  if(vPosition==0) vPosition=it~dT~PImax;
  else {
    vParam=loadmat(sprint("entry_prob_temp_spec",iSpec_label,".mat"))[][0];
    it=vPosition[0];
    dT=vPosition[1];
    PImax=vPosition[2];
    vNetwork=loadmat(sprint("entry_prob_temp_spec",iSpec_label,".mat"))[][1];
  }
  println("Initial position: ","%c",{"it","dT","PImax"},it~dT~PImax);
  
  do{
    time0=timer();
    /* Step 1: Simulate profits */
    sim_profit(&vSimPI,&mParam_range,vParam,iSequential_update,Sim);
    LocalMaxPI=maxc(vSimPI);
    vParam_max=mParam_range[][0];
    //if(iGreedy) greedy_search(&vParam_max,&LocalMaxPI);
    parallel for(i=0;i<rows(vParam);i++) vParam[i]=vParam[i].*(1-LR[i])+LR[i].*vParam_max[i]; /* Update network: current probability vector + step size corresponding to best network */
    if(LocalMaxPI>PImax) {
      vNetwork=vParam_max; /* Update best network */
      dObj=LocalMaxPI-PImax; /* change in objective function */
      PImax=LocalMaxPI; /* Update max profit */
      dT=0; /* reset counter */
    }
    else dT+=1;
    /* Step 2: Mutation */
    if(iDummy_SC) {
      parallel for(i=0;i<rows(vParam);i++) if(ranu(1,1)<MUT_PROB) vParam[i]=vParam[i]*(1-MUT_SHIFT)+ranu(1,1)*MUT_SHIFT;
    }
    else {
      parallel for(i=0;i<N;i++) if(ranu(1,1)<MUT_PROB) vParam[i]=vParam[i]*(1-MUT_SHIFT)+ranu(1,1)*MUT_SHIFT;
    }
    println("%c",{"cpu sec","gen","dT","dObj","E[PI]","Max PI","nbFC","nbSC","min","0.25","0.5","0.75","max"},
	    ((timer()-time0)/100)~it~dT~dObj~meanc(vSimPI)~PImax~sumc(vParam[:N-1])~sumc(vParam[N:])~quantilec(vParam[:N-1],<0,1/4,1/2,3/4,1>)');
    it+=1;
    /* Save current iteration */    
    savemat(sprint("entry_prob_temp_spec",iSpec_label,".mat"),vParam~vNetwork);
    savemat(sprint("position_temp_spec",iSpec_label,".mat"),it~(dT+1)~PImax);
  }while(dT<MAXIT);

  /* Greedy iterations */
  greedy_search(&vNetwork,&LocalMaxPI);

  /* Report results */
  aSolution_network[0]=vNetwork;
  aSolution_profits[0]=PImax;
  
  return 1;
}