#delimit;
cd "~/Dropbox/Amazon_FC/Data/Processed/Swap_list";
local tables_dir "~/Dropbox/Amazon_FC/Tables_and_Figures";

local taxdelay taxdelay2;
use Swap_diff_components_`taxdelay'_NLS_variety1_demandspec2.dta, clear;
drop if swapid==0;
format * %9.6g;

merge 1:1 swapid using Swap_list, 
	keepusing(fc_idA fc_idB state_codeA sq_ftA odateA oyearA cyearA typeidA  
		state_codeB sq_ftB odateB oyearB cyearB typeidB );
			
drop _merge;

/* Select inequalities */
keep if typeidA==2 | typeidA==4 | typeidA==9;
drop if cyearB<2400;
drop if oyearA==1999;
drop if oyearB==1999;
drop if swap_distance==0;

gen dummy_sc=typeidA==9;
gen dummy_fc=1-dummy_sc;

gen operating_profit=net_revenue-labor_cost-land_cost;
bysort fc_idA: egen count_swaps=count(fc_idB);
sort fc_idA fc_idB;

save Swap_estimation_sample, replace;

tabstat swapid year  net_revenue labor_cost land_cost density_cap density network_distance network_density 
  ship_int ship_dist ship_vi ship_vi_dist avg_wage avg_rent avg_density hh_dist 
  hh_vi hh_dist_nearest hh_vi_nearest swap_year0 swap_dyear swap_distance 
  swap_capacity swap_type tax_avg tax_hh, stat(median iqr)  columns(statistics);
gen avg_cost=(avg_rent+avg_wage)/2;  
foreach var in avg_density avg_cost avg_rent avg_wage hh_dist hh_vi tax_avg swap_capacity {; 
	egen iqr=iqr(`var');
	gen Sim_`var'=abs(`var')<iqr if `var'!=hh_dist;
	replace Sim_`var'=abs(`var')<iqr if `var'==hh_dist;
	replace Sim_`var'=abs(`var')<iqr if `var'==swap_capacity;
	gen I_`var'=`var'>0;
	gen D_`var'=`var'<0;
	drop iqr;
	};
replace Sim_tax_avg=tax_avg==0;


gen Orders0tax=I_tax_avg*Sim_hh_dist*Sim_hh_vi;
gen Orders1tax=D_tax_avg*Sim_hh_dist*Sim_hh_vi;

gen Dist1cost=I_hh_dist*D_avg_cost*Sim_tax_avg*Sim_hh_vi*Sim_avg_density; /* Upper bound */
gen Dist0cost=D_hh_dist*I_avg_cost*Sim_tax_avg*Sim_hh_vi*Sim_avg_density; /* Lower bound */
gen Dist1tax=I_hh_dist*D_tax_avg*Sim_avg_cost*Sim_hh_vi*Sim_avg_density; /* Upper bound */
gen Dist0tax=D_hh_dist*I_tax_avg*Sim_avg_cost*Sim_hh_vi*Sim_avg_density; /* Lower bound */
gen Dist1comb=I_hh_dist*D_avg_cost*(Sim_tax_avg+D_tax_avg)*Sim_hh_vi*Sim_avg_density; /* Upper bound */
gen Dist0comb=D_hh_dist*I_avg_cost*(Sim_tax_avg+I_tax_avg)*Sim_hh_vi*Sim_avg_density; /* Lower bound */

gen VI0cost=I_hh_vi*I_avg_cost*Sim_tax_avg*Sim_hh_dist*Sim_avg_density; /* Lower bound */
gen VI1cost=D_hh_vi*D_avg_cost*Sim_tax_avg*Sim_hh_dist*Sim_avg_density; /* Upper bound */
gen VI0tax=I_hh_vi*I_tax_avg*Sim_avg_cost*Sim_hh_dist*Sim_avg_density; /* Lower bound */
gen VI1tax=D_hh_vi*D_tax_avg*Sim_avg_cost*Sim_hh_dist*Sim_avg_density; /* Upper bound */
gen VI0comb=I_hh_vi*I_avg_cost*(Sim_tax_avg+I_tax_avg)*Sim_hh_dist*Sim_avg_density; /* Lower bound */
gen VI1comb=D_hh_vi*D_avg_cost*(Sim_tax_avg+D_tax_avg)*Sim_hh_dist*Sim_avg_density; /* Upper bound */

gen VI1Dist=I_hh_dist*I_hh_vi*Sim_tax_avg*Sim_avg_cost*Sim_avg_density; /* Upper bound */
gen VI0Dist=D_hh_dist*D_hh_vi*Sim_tax_avg*Sim_avg_cost*Sim_avg_density; /* Lower bound */

gen Dens1Dist=D_avg_density*I_hh_dist*Sim_tax_avg*Sim_avg_cost*Sim_hh_vi; /* Upper bound */
gen Dens0Dist=I_avg_density*D_hh_dist*Sim_tax_avg*Sim_avg_cost*Sim_hh_vi; /* Lower bound */

gen Dens0VI=I_avg_density*I_hh_vi*Sim_tax_avg*Sim_avg_cost*Sim_hh_dist; /* Lower bound */
gen Dens1VI=D_avg_density*D_hh_vi*Sim_tax_avg*Sim_avg_cost*Sim_hh_dist; /* Upper bound */


foreach  i in 0 1{;
	tabstat VI`i'cost VI`i'tax VI`i'comb Dist`i'cost Dist`i'tax Dist`i'comb 
		Dens`i'Dist Dens`i'VI VI`i'Dist, stat(mean);
	};


cd `tables_dir';

foreach stat in mean sd {;
foreach y in operating_profit ship_int ship_dist ship_vi density_cap {;
preserve;
foreach var in Orders0tax Orders1tax VI0cost VI1cost VI0tax VI1tax VI1comb VI0comb Dist0cost Dist1cost 
	Dist0tax Dist1tax Dist0comb Dist1comb Dens0Dist Dens1Dist 
	Dens0VI Dens1VI VI0Dist VI1Dist {;
	replace `var'=. if `var'==0;
	replace `var'=`var'*`y';	
	rename `var' `stat'`var';
	};
keep `stat'* ;
sum;
collapse (`stat') `stat'* ;
gen id=1;
reshape long `stat', i(id) j(IV) string;
rename `stat' `stat'_`y';
list;
save `stat'_swap_`y', replace;
restore;
};
};


preserve;
foreach var in Orders0tax Orders1tax VI0cost VI1cost VI0tax VI1tax VI1comb VI0comb Dist0cost Dist1cost 
	Dist0tax Dist1tax Dist0comb Dist1comb Dens0Dist Dens1Dist 
	Dens0VI Dens1VI VI0Dist VI1Dist {;
	replace `var'=. if `var'==0;
	replace `var'=`var';	
	rename `var' N`var';
	};
keep N* ;
sum;
collapse (sum) N* ;
gen id=1;
reshape long N, i(id) j(IV) string;
save Count_swap, replace;
restore;

/* Combined files */
use Count_swap, clear;
foreach y in operating_profit ship_dist ship_vi density_cap ship_int {;
	merge 1:1 id IV using mean_swap_`y';
	drop _merge;
	};
foreach y in operating_profit ship_dist ship_vi density_cap ship_int {;
	merge 1:1 id IV using sd_swap_`y';
	drop _merge;
	};

gen Bounds="Upper" if strpos(IV,"1");
replace Bounds="Lower" if strpos(IV,"0");
split IV, parse(0) gen(IV0);
split IV, parse(1) gen(IV1);

gen Parameters=IV01 if Bounds=="Lower";
replace Parameters=IV11 if Bounds=="Upper";
gen Variables=IV02 if Bounds=="Lower";
replace Variables=IV12 if Bounds=="Upper";

drop id;
save Swap_IV_statistics, replace;

local varlist Parameters Bounds Variables N mean_operating_profit mean_ship_dist 
	mean_ship_vi mean_density_cap mean_ship_int sd_operating_profit sd_ship_dist sd_ship_vi sd_density_cap sd_ship_int;
sum `varlist';	

outsheet `varlist' using Swap_IV_statistics_taxdelay2.csv, comma replace;
