capture log close
set more off
clear
// Project: Nexus Tax Laws and Economies of Density in E-Commerce: A Study of Amazon’s Fulfillment Center Network
// Authors: Houde, Newberry, Seim
// Created by: Peter Newberry
// Date: 05/2022
// Description: Creates Figure 5 in the main body of the paper. 
// Data sets needed: state_centroids.dta: mean of county centroids from county_panel.dta. could use any version of state centroids as it only places where the tax rates are visible. 
//					 county_panel.dta: county demographics, etc., created by authors
//					 Network_maximum_yearYYYY.dta: network solution w and w/out nexus laws. 
// Author notes: based on author do-file "cf_mapping.do"
*********************************************************
*********************************************************
*********************************************************
*********************************************************
*********************************************************
*********************************************************
*********************************************************

// create temporary file that has average tax rates in a state and the lat/lon of the centroid of the state.  
use "county_panel.dta", clear
egen tot_hh_state=sum(nb_hh), by(state year)
egen state_tax_rate=sum(nb_hh*tax/tot_hh_state), by(state year)
contract state year state_code tot_hh_state state_tax_rate
drop _freq
merge m:1 state_code using "state_centroids.dta"
replace state=trim(state)

// move centroids so they avoid the dots and are visible. this is mostly trial and error. 
replace state_lat=state_lat-2 if state=="California"
replace state_lon=state_lon+3 if state=="California"
replace state_lon=state_lon+1.5 if state=="Nevada"
replace state_lat=state_lat+2 if state=="Arizona"
replace state_lat=state_lat+1 if state=="Oklahoma"
replace state_lat=state_lat+1.2 if state=="Indiana"
replace state_lon=state_lon-.6 if state=="Louisiana"
replace state_lon=state_lon-2 if state=="Tennessee"
replace state_lon=state_lon+1.8 if state=="Kentucky"
replace state_lon=state_lon-.8 if state=="Ohio"
replace state_lat=state_lat+.6 if state=="Ohio"
replace state_lon=state_lon+.8 if state=="South Carolina"
replace state_lat=state_lat-.6 if state=="South Carolina"
replace state_lon=state_lon+1.6 if state=="North Carolina"
replace state_lat=state_lat+.5 if state=="North Carolina"
replace state_lon=state_lon-.5 if state=="Virginia"
replace state_lat=state_lat-.4 if state=="Virginia"
replace state_lon=state_lon-.5 if state=="Pennsylvania"
replace state_lon=state_lon+2.3 if state=="Maryland"
replace state_lat=state_lat-1 if state=="Maryland"
replace state_lon=state_lon+1.4 if state=="Delaware"
replace state_lat=state_lat-.4 if state=="Delaware"
replace state_lon=state_lon+1.4 if state=="New Jersey"
replace state_lat=state_lat-.2 if state=="New Jersey"
replace state_lon=state_lon+1.4 if state=="Connecticut"
replace state_lat=state_lat-.6 if state=="Connecticut"
replace state_lon=state_lon+2.2 if state=="Rhode Island"
replace state_lat=state_lat-.3 if state=="Rhode Island"
replace state_lon=state_lon+1.5 if state=="Massachusetts"
replace state_lat=state_lat+.4 if state=="Massachusetts"
replace state_lat=state_lat+.5 if state=="Vermont"
replace state_lon=state_lon+.1 if state=="Vermont"
replace state_lat=state_lat+.8 if state=="Maine"
replace state_lon=state_lon-.05 if state=="New Hampshire"
replace state_lat=state_lat+.6 if state=="Florida"
replace state_lat=state_lat+1 if state=="Minnesota"
replace state_lon=state_lon+1.7 if state=="Florida"
replace state_lon=state_lon-.2 if state=="Iowa"
replace state_lat=state_lat+.5 if state=="Washington"
replace state_tax_rate=. if state=="District of Columbia"
drop _merge
save "Dump/temp.dta", replace

// create tax rates for each year becase the maps are year by year. 
preserve
keep if year==2000
replace state_tax_rate=100*state_tax_rate
format state_tax_rate %9.2f
save "Dump/tax2000.dta", replace
restore
preserve
keep if year==2006
replace state_tax_rate=100*state_tax_rate
format state_tax_rate %9.2f
save "Dump/tax2006.dta", replace
restore
preserve
keep if year==2012
replace state_tax_rate=100*state_tax_rate
format state_tax_rate %9.2f
save "Dump/tax2012.dta", replace
restore
preserve
keep if year==2018
replace state_tax_rate=100*state_tax_rate
format state_tax_rate %9.2f
save "Dump/tax2018.dta", replace
restore


// create a file that has locations of FCs and SCs in each of the counterfactual scenarios. 
use "Network_maximum_year1999.dta", clear
append using "Network_maximum_year2006.dta"
append using "Network_maximum_year2012.dta"
append using "Network_maximum_year2018.dta"
drop taxdelay
rename nb_fc0 nb_fc

// keep if there are FCs in a given location.
keep if nb_fc>0

// no nexus laws. 
gen taxdelay=0
keep nb_fc cluster_lat cluster_long county_code state_abbrev year taxdelay cluster_id

save "temp.dta", replace

use "Network_maximum_year1999.dta", clear
append using "Network_maximum_year2006.dta"
append using "Network_maximum_year2012.dta"
append using "Network_maximum_year2018.dta"
drop taxdelay
rename nb_fc1 nb_fc
// keep if there are FCs in a given location.
keep if nb_fc>0


// there are nexus laws. 
gen taxdelay=1
keep nb_fc cluster_lat cluster_long county_code state_abbrev year taxdelay  cluster_id

//apend previous data
append using "temp.dta"
save "temp.dta", replace


use "Network_maximum_year1999.dta", clear
append using "Network_maximum_year2006.dta"
append using "Network_maximum_year2012.dta"
append using "Network_maximum_year2018.dta"
drop taxdelay
rename nb_sc0 nb_sc
// keep if there are SCs in a given location.
keep if nb_sc>0

// no nexus laws. 
gen taxdelay=0
keep nb_sc cluster_lat cluster_long county_code state_abbrev year taxdelay  cluster_id
gen sc_dum=1

//apend previous data
append using "temp.dta"
save "temp.dta", replace

use "Network_maximum_year1999.dta", clear
append using "Network_maximum_year2006.dta"
append using "Network_maximum_year2012.dta"
append using "Network_maximum_year2018.dta"
drop taxdelay
rename nb_sc1 nb_sc
// keep if there are SCs in a given location.
keep if nb_sc>0

// there are nexus laws. 
gen taxdelay=1
keep nb_sc cluster_lat cluster_long county_code state_abbrev year taxdelay  cluster_id
gen sc_dum=1

//apend previous data
append using "temp.dta"

replace sc_dum=0 if sc_dum==.

drop nb*

// shift location of the dots slightly
replace cluster_lon=1.003*cluster_lon if taxdelay==1


// create data each year which will be come the dots on the graph for the locations. 
preserve
keep if year==1999
keep if sc_dum==0
// re define what tax delay means.
replace taxdelay=1-taxdelay 
save "cf_data99_fc.dta", replace
restore

preserve
keep if year==2006
keep if sc_dum==0
// re define what tax delay means.
replace taxdelay=1-taxdelay
save "cf_data06_fc.dta", replace
restore

preserve
keep if year==2012
keep if sc_dum==0
// re define what tax delay means.
replace taxdelay=1-taxdelay
save "cf_data12_fc.dta", replace
restore

preserve
keep if year==2018
keep if sc_dum==0
// re define what tax delay means.
replace taxdelay=1-taxdelay
save "cf_data18_fc.dta", replace
restore

// append data together so we have one data set for the scenario with nexus laws
preserve
use "cf_data99_fc.dta", clear
append using "cf_data06_fc.dta"
append using "cf_data12_fc.dta"
append using "cf_data18_fc.dta"
keep if taxdelay==0

egen min_year=min(year), by(cluster_id)
contract cluster_id min_year cluster_lat cluster_lon
rename min_year year
gen point_size=0
replace point_size=23 if year==1999
replace point_size=16 if year==2006
replace point_size=8 if year==2012
replace point_size=1 if year==2018
save "cf_data_fc_dist_first.dta", replace
restore


// append data together so we have one data set for the scenario without nexus laws
preserve
use "cf_data99_fc.dta", clear
append using "cf_data06_fc.dta"
append using "cf_data12_fc.dta"
append using "cf_data18_fc.dta"
keep if taxdelay==1

egen min_year=min(year), by(cluster_id)
contract cluster_id min_year cluster_lat cluster_lon
rename min_year year
gen point_size=0
replace point_size=23 if year==1999
replace point_size=16 if year==2006
replace point_size=8 if year==2012
replace point_size=1 if year==2018
save "cf_data_fc_ndist_first.dta", replace
restore

erase "temp.dta"


//open shape file
shp2dta using "cb_2013_us_state_500k", database(st_data) coordinates(st_coordinates) genid(id) replace
use st_data, clear
rename STUSPS state
destring GEOID, replace
rename GEOID state_code

// merge in tax rates and lat/lon of states
merge 1:m state_code using "Dump/temp.dta"
keep if _merge==3

//redefine households 
rename tot_hh_state households
replace households=households/100000
replace households=round(households, .1)

// with nexus laws (top of figure) and in color
spmap households using "st_coordinates.dta" if year==2018 & id!=28 & id!=29 & id!=32 & id!=37 & id!=40 & id!=54 & id!=55, legend(size(14pt)) label(data("Dump/tax2018.dta") x(state_lon) y(state_lat) label(state_tax_rate) size(3))  fcolor(Blues) id(id) clmethod(custom) clbreaks(0 7.5 17.5 27.3 130.2) legend(label(2 "2.3-7.5") label(3 "7.5-17.5") label(4 "17.5-27.3") label(5 "27.3-130.2")) legenda(off)  legtitle("Households (100k)") legstyle(3) point(data("Data/Processed/Intermediate/cf_data_fc_dist_first.dta") x(cluster_lon) y(cluster_lat) by(year) ocolor(black black black black) osize(medium medium medium medium) fcolor(red orange yellow lime) legenda(on) size(large large large large) legtitle("First Entry Year") )
graph export "cf_max_fc_dist_first_color.png", replace

// with nexus laws (top of figure) and in B/W
spmap households using "st_coordinates.dta" if year==2018 & id!=28 & id!=29 & id!=32 & id!=37 & id!=40 & id!=54 & id!=55, legend(size(14pt)) label(data("Dump/tax2018.dta") x(state_lon) y(state_lat) label(state_tax_rate) size(3))  fcolor(gs16 gs14 gs11 gs8) id(id) clmethod(custom) clbreaks(0 7.5 17.5 27.3 130.2) legend(label(2 "2.3-7.5") label(3 "7.5-17.5") label(4 "17.5-27.3") label(5 "27.3-130.2"))  legenda(off)   legtitle("Households (100k)") legstyle(3) point(data("Data/Processed/Intermediate/cf_data_fc_dist_first.dta") x(cluster_lon) y(cluster_lat) by(year) ocolor(black black black black) osize(medium medium medium medium) fcolor(gs16 gs11 gs6 gs1) legenda(on) size(large large large large) legtitle("First Entry Year") )
graph export "cf_max_fc_dist_first.png", replace

// without nexus laws (bottom of figure) and in color
spmap households using "st_coordinates.dta" if year==2018 & id!=28 & id!=29 & id!=32 & id!=37 & id!=40 & id!=54 & id!=55, legend(size(14pt)) label(data("Dump/tax2018.dta") x(state_lon) y(state_lat) label(state_tax_rate) size(3))  fcolor(Blues) id(id) clmethod(custom) clbreaks(0 7.5 17.5 27.3 130.2) legend(label(2 "2.3-7.5") label(3 "7.5-17.5") label(4 "17.5-27.3") label(5 "27.3-130.2")) legenda(off) legtitle("Households (100k)") legstyle(3) point(data("Data/Processed/Intermediate/cf_data_fc_ndist_first.dta") x(cluster_lon) y(cluster_lat) by(year) ocolor(black black black black) osize(medium medium medium medium) fcolor(red orange yellow  lime) legenda(on) size(large large large large) legtitle("First Entry Year") )
graph export "cf_max_fc_ndist_first_color.png", replace

// without nexus laws (bottom of figure) and in B/W
spmap households using "st_coordinates.dta" if year==2018 & id!=28 & id!=29 & id!=32 & id!=37 & id!=40 & id!=54 & id!=55, legend(size(14pt)) label(data("Dump/tax2018.dta") x(state_lon) y(state_lat) label(state_tax_rate) size(3))  fcolor(gs16 gs14 gs11 gs8) id(id) clmethod(custom) clbreaks(0 7.5 17.5 27.3 130.2) legend(label(2 "2.3-7.5") label(3 "7.5-17.5") label(4 "17.5-27.3") label(5 "27.3-130.2")) legenda(off)   legtitle("Households (100k)") legstyle(3) point(data("Data/Processed/Intermediate/cf_data_fc_ndist_first.dta") x(cluster_lon) y(cluster_lat) by(year) ocolor(black black black black) osize(medium medium medium medium) fcolor(gs16 gs11 gs6 gs1) legenda(on) size(large large large large) legtitle("First Entry Year") )
graph export "cf_max_fc_ndist_first.png", replace


